@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -16,6 +16,7 @@
 from nautilus_trader.adapters.binance.common.constants import BINANCE_VENUE
 from nautilus_trader.adapters.binance.common.enums import BinanceAccountType
 from nautilus_trader.adapters.binance.common.enums import BinanceKeyType
+from nautilus_trader.adapters.binance.common.symbol import BinanceSymbol
 from nautilus_trader.config import LiveDataClientConfig
 from nautilus_trader.config import LiveExecClientConfig
 from nautilus_trader.config import PositiveFloat
@@ -110,6 +111,9 @@ class BinanceExecClientConfig(LiveExecClientConfig, frozen=True):
         If Binance Futures hedging position IDs should be used.
         If False, then order event `position_id`(s) from the execution client will be `None`, which
         allows *virtual* positions with `OmsType.HEDGING`.
+    use_trade_lite: bool, default False
+        If TRADE_LITE events should be used.
+        If True, commissions will be calculated based on the instrument's details.
     treat_expired_as_canceled : bool, default False
         If the `EXPIRED` execution type is semantically treated as `CANCELED`.
         Binance treats cancels with certain combinations of order type and time in force as expired
@@ -120,6 +124,8 @@ class BinanceExecClientConfig(LiveExecClientConfig, frozen=True):
         The maximum number of times a submit, cancel or modify order request will be retried.
     retry_delay : PositiveFloat, optional
         The delay (seconds) between retries. Short delays with frequent retries may result in account bans.
+    futures_leverages : dict[BinanceSymbol, PositiveInt], optional
+        The initial leverage to be used for each symbol. It's applicable to futures only.
 
     Warnings
     --------
@@ -139,7 +145,9 @@ class BinanceExecClientConfig(LiveExecClientConfig, frozen=True):
     use_gtd: bool = True
     use_reduce_only: bool = True
     use_position_ids: bool = True
+    use_trade_lite: bool = False
     treat_expired_as_canceled: bool = False
     recv_window_ms: PositiveInt = 5_000
     max_retries: PositiveInt | None = None
     retry_delay: PositiveFloat | None = None
+    futures_leverages: dict[BinanceSymbol, PositiveInt] | None = None
