@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -13,11 +13,13 @@
 #  limitations under the License.
 # -------------------------------------------------------------------------------------------------
 
+from __future__ import annotations
+
 from enum import Enum
 from enum import unique
+from typing import TYPE_CHECKING
 
 from nautilus_trader.core.nautilus_pyo3 import PositionSide
-from nautilus_trader.model.data import BarType
 from nautilus_trader.model.enums import BarAggregation
 from nautilus_trader.model.enums import OrderSide
 from nautilus_trader.model.enums import OrderStatus
@@ -27,6 +29,10 @@ from nautilus_trader.model.enums import TriggerType
 from nautilus_trader.model.enums import time_in_force_to_str
 
 
+if TYPE_CHECKING:
+    from nautilus_trader.model.data import BarType
+
+
 def raise_error(error):
     raise error
 
@@ -47,6 +53,16 @@ class BybitMarginMode(Enum):
     PORTFOLIO_MARGIN = "PORTFOLIO_MARGIN"
 
 
+@unique
+class BybitPositionMode(Enum):
+    """
+    https://bybit-exchange.github.io/docs/v5/position/position-mode
+    """
+
+    MERGED_SINGLE = 0
+    BOTH_SIDES = 3
+
+
 @unique
 class BybitPositionIdx(Enum):
     # One-way mode position
@@ -116,6 +132,13 @@ class BybitPositionSide(Enum):
         raise RuntimeError(f"invalid position side, was {self}")
 
 
+@unique
+class BybitWsOrderRequestMsgOP(Enum):
+    CREATE = "order.create"
+    AMEND = "order.amend"
+    CANCEL = "order.cancel"
+
+
 @unique
 class BybitKlineInterval(Enum):
     MINUTE_1 = "1"
@@ -149,6 +172,7 @@ class BybitOrderStatus(Enum):
 
 @unique
 class BybitOrderSide(Enum):
+    UNKNOWN = ""  # It will be an empty string in some cases
     BUY = "Buy"
     SELL = "Sell"
 
