@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -13,13 +13,23 @@
 #  limitations under the License.
 # -------------------------------------------------------------------------------------------------
 
-from nautilus_trader.adapters.bybit.common.enums import BybitProductType
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
 from nautilus_trader.config import LiveDataClientConfig
 from nautilus_trader.config import LiveExecClientConfig
 from nautilus_trader.config import PositiveFloat
 from nautilus_trader.config import PositiveInt
 
 
+if TYPE_CHECKING:
+    from nautilus_trader.adapters.bybit.common.enums import BybitMarginMode
+    from nautilus_trader.adapters.bybit.common.enums import BybitPositionMode
+    from nautilus_trader.adapters.bybit.common.enums import BybitProductType
+    from nautilus_trader.adapters.bybit.common.symbol import BybitSymbol
+
+
 class BybitDataClientConfig(LiveDataClientConfig, frozen=True):
     """
     Configuration for ``BybitDataClient`` instances.
@@ -43,9 +53,8 @@ class BybitDataClientConfig(LiveDataClientConfig, frozen=True):
         If the client is connecting to the Bybit testnet API.
     update_instruments_interval_mins: PositiveInt or None, default 60
         The interval (minutes) between reloading instruments from the venue.
-    max_ws_reconnection_tries: int, default 3
-        The number of retries to reconnect the websocket connection if the
-        connection is broken.
+    recv_window_ms : PositiveInt, default 5000
+        The receive window (milliseconds) for Bybit HTTP requests.
 
     """
 
@@ -56,7 +65,7 @@ class BybitDataClientConfig(LiveDataClientConfig, frozen=True):
     demo: bool = False
     testnet: bool = False
     update_instruments_interval_mins: PositiveInt | None = 60
-    max_ws_reconnection_tries: int | None = 3
+    recv_window_ms: PositiveInt = 5_000
 
 
 class BybitExecClientConfig(LiveExecClientConfig, frozen=True):
@@ -73,11 +82,15 @@ class BybitExecClientConfig(LiveExecClientConfig, frozen=True):
         The Bybit API public key.
         If ``None`` then will source the `BYBIT_API_KEY` or
         `BYBIT_TESTNET_API_KEY` environment variables.
-    product_type : list[BybitProductType], optional
+    product_types : list[BybitProductType], optional
         The Bybit product type for the client.
         If None then will default to 'SPOT', you also cannot mix 'SPOT' with
         any other product type for execution, and it will use a `CASH` account
         type, vs `MARGIN` for the other derivative products.
+    base_url_ws_private : str, optional
+        The base URL for the `private` WebSocket client.
+    base_url_ws_trade : str, optional
+        The base URL for the `trade` WebSocket client.
     demo : bool, default False
         If the client is connecting to the Bybit demo API.
     testnet : bool, default False
@@ -85,13 +98,27 @@ class BybitExecClientConfig(LiveExecClientConfig, frozen=True):
     use_gtd : bool, default False
         If False, then GTD time in force will be remapped to GTC
         (this is useful if managing GTD orders locally).
+    use_ws_execution_fast : bool, default False
+        If use fast execution stream.
+    use_ws_trade_api : bool, default False
+        If the client is using websocket to send order requests.
+    use_http_batch_api : bool, default False
+        If the client is using http api to send batch order requests.
+        Effective only when `use_ws_trade_api` is set to `True`.
     max_retries : PositiveInt, optional
         The maximum number of times a submit, cancel or modify order request will be retried.
     retry_delay : PositiveFloat, optional
         The delay (seconds) between retries. Short delays with frequent retries may result in account bans.
-    max_ws_reconnection_tries: int, default 3
-        The number of retries to reconnect the websocket connection if the
-        connection is broken.
+    recv_window_ms : PositiveInt, default 5000
+        The receive window (milliseconds) for Bybit HTTP requests.
+    ws_trade_timeout_secs : float, default 5.0
+        The timeout for trade websocket messages.
+    futures_leverages : dict[BybitSymbol, PositiveInt], optional
+        The leverages for futures.
+    position_mode : BybitPositionMode, optional
+        The position mode for `USDT perpetual` and `Inverse futures`.
+    margin_mode : BybitMarginMode, optional
+        Set Margin Mode.
 
     Warnings
     --------
@@ -103,10 +130,18 @@ class BybitExecClientConfig(LiveExecClientConfig, frozen=True):
     api_secret: str | None = None
     product_types: list[BybitProductType] | None = None
     base_url_http: str | None = None
-    base_url_ws: str | None = None
+    base_url_ws_private: str | None = None
+    base_url_ws_trade: str | None = None
     demo: bool = False
     testnet: bool = False
     use_gtd: bool = False  # Not supported on Bybit
+    use_ws_execution_fast: bool = False
+    use_ws_trade_api: bool = False
+    use_http_batch_api: bool = False
     max_retries: PositiveInt | None = None
     retry_delay: PositiveFloat | None = None
-    max_ws_reconnection_tries: int | None = 3
+    recv_window_ms: PositiveInt = 5_000
+    ws_trade_timeout_secs: float | None = 5.0
+    futures_leverages: dict[BybitSymbol, PositiveInt] | None = None
+    position_mode: dict[BybitSymbol, BybitPositionMode] | None = None
+    margin_mode: BybitMarginMode | None = None
