@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -13,11 +13,12 @@
 #  limitations under the License.
 # -------------------------------------------------------------------------------------------------
 
-import asyncio
-from decimal import Decimal
+from __future__ import annotations
+
+from asyncio import TaskGroup
+from typing import TYPE_CHECKING
 
 import msgspec
-import pandas as pd
 
 from nautilus_trader.adapters.bybit.common.constants import BYBIT_VENUE
 from nautilus_trader.adapters.bybit.common.credentials import get_api_key
@@ -31,39 +32,25 @@ from nautilus_trader.adapters.bybit.common.enums import BybitTimeInForce
 from nautilus_trader.adapters.bybit.common.enums import BybitTpSlMode
 from nautilus_trader.adapters.bybit.common.enums import BybitTriggerDirection
 from nautilus_trader.adapters.bybit.common.symbol import BybitSymbol
-from nautilus_trader.adapters.bybit.config import BybitExecClientConfig
 from nautilus_trader.adapters.bybit.endpoints.trade.batch_cancel_order import BybitBatchCancelOrder
 from nautilus_trader.adapters.bybit.endpoints.trade.batch_place_order import BybitBatchPlaceOrder
 from nautilus_trader.adapters.bybit.http.account import BybitAccountHttpAPI
-from nautilus_trader.adapters.bybit.http.client import BybitHttpClient
 from nautilus_trader.adapters.bybit.http.errors import BybitError
 from nautilus_trader.adapters.bybit.http.errors import should_retry
-from nautilus_trader.adapters.bybit.providers import BybitInstrumentProvider
 from nautilus_trader.adapters.bybit.schemas.common import BYBIT_PONG
 from nautilus_trader.adapters.bybit.schemas.ws import BybitWsAccountExecution
+from nautilus_trader.adapters.bybit.schemas.ws import BybitWsAccountExecutionFast
+from nautilus_trader.adapters.bybit.schemas.ws import BybitWsAccountExecutionFastMsg
 from nautilus_trader.adapters.bybit.schemas.ws import BybitWsAccountExecutionMsg
 from nautilus_trader.adapters.bybit.schemas.ws import BybitWsAccountOrderMsg
-from nautilus_trader.adapters.bybit.schemas.ws import BybitWsAccountPositionMsg
 from nautilus_trader.adapters.bybit.schemas.ws import BybitWsAccountWalletMsg
 from nautilus_trader.adapters.bybit.schemas.ws import BybitWsMessageGeneral
-from nautilus_trader.adapters.bybit.schemas.ws import BybitWsSubscriptionMsg
 from nautilus_trader.adapters.bybit.websocket.client import BybitWebSocketClient
-from nautilus_trader.cache.cache import Cache
-from nautilus_trader.common.component import LiveClock
-from nautilus_trader.common.component import MessageBus
 from nautilus_trader.common.enums import LogColor
+from nautilus_trader.common.enums import LogLevel
 from nautilus_trader.core.correctness import PyCondition
 from nautilus_trader.core.datetime import millis_to_nanos
 from nautilus_trader.core.uuid import UUID4
-from nautilus_trader.execution.messages import BatchCancelOrders
-from nautilus_trader.execution.messages import CancelAllOrders
-from nautilus_trader.execution.messages import CancelOrder
-from nautilus_trader.execution.messages import ModifyOrder
-from nautilus_trader.execution.messages import SubmitOrder
-from nautilus_trader.execution.messages import SubmitOrderList
-from nautilus_trader.execution.reports import FillReport
-from nautilus_trader.execution.reports import OrderStatusReport
-from nautilus_trader.execution.reports import PositionStatusReport
 from nautilus_trader.live.execution_client import LiveExecutionClient
 from nautilus_trader.live.retry import RetryManagerPool
 from nautilus_trader.model.enums import AccountType
@@ -92,7 +79,32 @@ from nautilus_trader.model.orders import StopLimitOrder
 from nautilus_trader.model.orders import StopMarketOrder
 from nautilus_trader.model.orders import TrailingStopLimitOrder
 from nautilus_trader.model.orders import TrailingStopMarketOrder
-from nautilus_trader.model.position import Position
+
+
+if TYPE_CHECKING:
+    import asyncio
+
+    from nautilus_trader.adapters.bybit.common.enums import BybitPositionMode
+    from nautilus_trader.adapters.bybit.config import BybitExecClientConfig
+    from nautilus_trader.adapters.bybit.http.client import BybitHttpClient
+    from nautilus_trader.adapters.bybit.providers import BybitInstrumentProvider
+    from nautilus_trader.cache.cache import Cache
+    from nautilus_trader.common.component import LiveClock
+    from nautilus_trader.common.component import MessageBus
+    from nautilus_trader.execution.messages import BatchCancelOrders
+    from nautilus_trader.execution.messages import CancelAllOrders
+    from nautilus_trader.execution.messages import CancelOrder
+    from nautilus_trader.execution.messages import GenerateFillReports
+    from nautilus_trader.execution.messages import GenerateOrderStatusReport
+    from nautilus_trader.execution.messages import GenerateOrderStatusReports
+    from nautilus_trader.execution.messages import GeneratePositionStatusReports
+    from nautilus_trader.execution.messages import ModifyOrder
+    from nautilus_trader.execution.messages import SubmitOrder
+    from nautilus_trader.execution.messages import SubmitOrderList
+    from nautilus_trader.execution.reports import FillReport
+    from nautilus_trader.execution.reports import OrderStatusReport
+    from nautilus_trader.execution.reports import PositionStatusReport
+    from nautilus_trader.model.position import Position
 
 
 class BybitExecutionClient(LiveExecutionClient):
@@ -115,8 +127,10 @@ class BybitExecutionClient(LiveExecutionClient):
         The instrument provider.
     product_types : list[BybitProductType]
         The product types for the client.
-    base_url_ws : str
-        The base URL for the WebSocket client.
+    base_url_ws_private : str
+        The base URL for the `private` WebSocket client.
+    base_url_ws_trade : str
+        The base URL for the `trade` WebSocket client.
     config : BybitExecClientConfig
         The configuration for the client.
     name : str, optional
@@ -133,7 +147,8 @@ class BybitExecutionClient(LiveExecutionClient):
         clock: LiveClock,
         instrument_provider: BybitInstrumentProvider,
         product_types: list[BybitProductType],
-        base_url_ws: str,
+        base_url_ws_private: str,
+        base_url_ws_trade: str,
         config: BybitExecClientConfig,
         name: str | None,
     ) -> None:
@@ -160,36 +175,75 @@ class BybitExecutionClient(LiveExecutionClient):
         # Configuration
         self._product_types = product_types
         self._use_gtd = config.use_gtd
+        self._use_ws_trade_api = config.use_ws_trade_api
+        self._use_ws_execution_fast = config.use_ws_execution_fast
+        self._use_http_batch_api = config.use_http_batch_api
+
+        self._futures_leverages = config.futures_leverages
+        self._margin_mode = config.margin_mode
+        self._position_mode = config.position_mode
+
         self._log.info(f"Account type: {account_type_to_str(account_type)}", LogColor.BLUE)
         self._log.info(f"Product types: {[p.value for p in product_types]}", LogColor.BLUE)
         self._log.info(f"{config.use_gtd=}", LogColor.BLUE)
+        self._log.info(f"{config.use_ws_execution_fast=}", LogColor.BLUE)
+        self._log.info(f"{config.use_ws_trade_api=}", LogColor.BLUE)
+        self._log.info(f"{config.use_http_batch_api=}", LogColor.BLUE)
         self._log.info(f"{config.max_retries=}", LogColor.BLUE)
         self._log.info(f"{config.retry_delay=}", LogColor.BLUE)
+        self._log.info(f"{config.recv_window_ms=:_}", LogColor.BLUE)
+        self._log.info(f"{config.ws_trade_timeout_secs=}", LogColor.BLUE)
+        self._log.info(f"{config.futures_leverages=}", LogColor.BLUE)
+        self._log.info(f"{config.margin_mode=}", LogColor.BLUE)
+        self._log.info(f"{config.position_mode=}", LogColor.BLUE)
 
         self._enum_parser = BybitEnumParser()
 
         account_id = AccountId(f"{name or BYBIT_VENUE.value}-UNIFIED")
         self._set_account_id(account_id)
 
-        # WebSocket API
-        self._ws_client = BybitWebSocketClient(
-            clock=clock,
-            handler=self._handle_ws_message,
-            handler_reconnect=None,
-            base_url=base_url_ws,
-            is_private=True,
-            api_key=config.api_key or get_api_key(config.demo, config.testnet),
-            api_secret=config.api_secret or get_api_secret(config.demo, config.testnet),
-            loop=loop,
-            max_reconnection_tries=config.max_ws_reconnection_tries,
-        )
-
         # HTTP API
         self._http_account = BybitAccountHttpAPI(
             client=client,
             clock=clock,
         )
 
+        # WebSocket private client
+        self._ws_private_client = BybitWebSocketClient(
+            clock=clock,
+            handler=self._handle_ws_message_private,
+            handler_reconnect=None,
+            base_url=base_url_ws_private,
+            is_private=True,
+            api_key=config.api_key or get_api_key(config.demo, config.testnet),
+            api_secret=config.api_secret or get_api_secret(config.demo, config.testnet),
+            loop=loop,
+        )
+
+        # WebSocket trade client
+        self._order_single_client: BybitWebSocketClient | BybitAccountHttpAPI
+        self._order_batch_client: BybitWebSocketClient | BybitAccountHttpAPI
+        if self._use_ws_trade_api:
+            self._ws_order_client = BybitWebSocketClient(
+                clock=clock,
+                handler=self._handle_ws_message_trade,
+                handler_reconnect=None,
+                base_url=base_url_ws_trade,
+                is_trade=True,
+                api_key=config.api_key or get_api_key(config.demo, config.testnet),
+                api_secret=config.api_secret or get_api_secret(config.demo, config.testnet),
+                loop=loop,
+                ws_trade_timeout_secs=config.ws_trade_timeout_secs,
+            )
+            self._order_single_client = self._ws_order_client
+            if config.use_http_batch_api:
+                self._order_batch_client = self._http_account
+            else:
+                self._order_batch_client = self._ws_order_client
+        else:
+            self._order_single_client = self._http_account
+            self._order_batch_client = self._http_account
+
         # Order submission
         self._submit_order_methods = {
             OrderType.MARKET: self._submit_market_order,
@@ -201,20 +255,31 @@ class BybitExecutionClient(LiveExecutionClient):
             OrderType.TRAILING_STOP_MARKET: self._submit_trailing_stop_market,
         }
 
+        self._batch_order_create_handlers = {
+            OrderType.MARKET: self._create_market_batch_order,
+            OrderType.LIMIT: self._create_limit_batch_order,
+            OrderType.LIMIT_IF_TOUCHED: self._create_limit_if_touched_batch_order,
+            OrderType.STOP_MARKET: self._create_stop_market_batch_order,
+            OrderType.MARKET_IF_TOUCHED: self._create_market_if_touched_batch_order,
+        }
+
         # Decoders
         self._decoder_ws_msg_general = msgspec.json.Decoder(BybitWsMessageGeneral)
-        self._decoder_ws_subscription = msgspec.json.Decoder(BybitWsSubscriptionMsg)
+        # self._decoder_ws_subscription = msgspec.json.Decoder(BybitWsSubscriptionMsg)
 
         self._decoder_ws_account_order_update = msgspec.json.Decoder(BybitWsAccountOrderMsg)
         self._decoder_ws_account_execution_update = msgspec.json.Decoder(BybitWsAccountExecutionMsg)
-        self._decoder_ws_account_position_update = msgspec.json.Decoder(BybitWsAccountPositionMsg)
+        self._decoder_ws_account_execution_fast_update = msgspec.json.Decoder(
+            BybitWsAccountExecutionFastMsg,
+        )
+        # self._decoder_ws_account_position_update = msgspec.json.Decoder(BybitWsAccountPositionMsg)
         self._decoder_ws_account_wallet_update = msgspec.json.Decoder(BybitWsAccountWalletMsg)
 
         # Hot caches
         self._instrument_ids: dict[str, InstrumentId] = {}
         self._pending_trailing_stops: dict[ClientOrderId, Order] = {}
 
-        self._retry_manager_pool = RetryManagerPool(
+        self._retry_manager_pool = RetryManagerPool[None](
             pool_size=100,
             max_retries=config.max_retries or 0,
             retry_delay_secs=config.retry_delay or 0.0,
@@ -227,13 +292,24 @@ class BybitExecutionClient(LiveExecutionClient):
         await self._instrument_provider.initialize()
         await self._update_account_state()
 
-        await self._ws_client.connect()
-        await self._ws_client.subscribe_executions_update()
-        await self._ws_client.subscribe_orders_update()
-        await self._ws_client.subscribe_wallet_update()
+        await self._ws_private_client.connect()
+
+        await self._ws_private_client.subscribe_orders_update()
+        await self._ws_private_client.subscribe_wallet_update()
+
+        if self._use_ws_execution_fast:
+            await self._ws_private_client.subscribe_executions_fast_update()
+        else:
+            await self._ws_private_client.subscribe_executions_update()
+
+        if self._use_ws_trade_api:
+            await self._ws_order_client.connect()
 
     async def _disconnect(self) -> None:
-        await self._ws_client.disconnect()
+        await self._ws_private_client.disconnect()
+
+        if self._use_ws_trade_api:
+            await self._ws_order_client.disconnect()
 
     def _stop(self) -> None:
         self._retry_manager_pool.shutdown()
@@ -242,11 +318,10 @@ class BybitExecutionClient(LiveExecutionClient):
 
     async def generate_order_status_reports(
         self,
-        instrument_id: InstrumentId | None = None,
-        start: pd.Timestamp | None = None,
-        end: pd.Timestamp | None = None,
-        open_only: bool = False,
+        command: GenerateOrderStatusReports,
     ) -> list[OrderStatusReport]:
+        instrument_id = command.instrument_id
+
         self._log.debug("Requesting OrderStatusReports...")
         reports: list[OrderStatusReport] = []
 
@@ -260,7 +335,7 @@ class BybitExecutionClient(LiveExecutionClient):
                 bybit_orders = await self._http_account.query_order_history(
                     product_type,
                     symbol,
-                    open_only,
+                    command.open_only,
                 )
                 for bybit_order in bybit_orders:
                     # Uncomment for development
@@ -291,16 +366,23 @@ class BybitExecutionClient(LiveExecutionClient):
 
         len_reports = len(reports)
         plural = "" if len_reports == 1 else "s"
-        self._log.info(f"Received {len(reports)} OrderStatusReport{plural}")
+        receipt_log = f"Received {len(reports)} OrderStatusReport{plural}"
+
+        if command.log_receipt_level == LogLevel.INFO:
+            self._log.info(receipt_log)
+        else:
+            self._log.debug(receipt_log)
 
         return reports
 
     async def generate_order_status_report(
         self,
-        instrument_id: InstrumentId,
-        client_order_id: ClientOrderId | None = None,
-        venue_order_id: VenueOrderId | None = None,
+        command: GenerateOrderStatusReport,
     ) -> OrderStatusReport | None:
+        instrument_id = command.instrument_id
+        client_order_id = command.client_order_id
+        venue_order_id = command.venue_order_id
+
         PyCondition.is_false(
             client_order_id is None and venue_order_id is None,
             "both `client_order_id` and `venue_order_id` were `None`",
@@ -316,9 +398,7 @@ class BybitExecutionClient(LiveExecutionClient):
                 client_order_id = None
 
         self._log.info(
-            f"Generating OrderStatusReport for "
-            f"{repr(client_order_id) if client_order_id else ''} "
-            f"{repr(venue_order_id) if venue_order_id else ''}",
+            f"Generating OrderStatusReport for {repr(client_order_id) if client_order_id else ''} {repr(venue_order_id) if venue_order_id else ''}",
         )
         try:
             bybit_symbol = BybitSymbol(instrument_id.symbol.value)
@@ -332,10 +412,10 @@ class BybitExecutionClient(LiveExecutionClient):
             if len(bybit_orders) == 0:
                 self._log.error(f"Received no order for {venue_order_id}")
                 return None
-            targetOrder = bybit_orders[0]
+            target_order = bybit_orders[0]
             if len(bybit_orders) > 1:
                 self._log.warning(f"Received more than one order for {venue_order_id}")
-                targetOrder = bybit_orders[0]
+                target_order = bybit_orders[0]
 
             order_link_id = bybit_orders[0].orderLinkId
             client_order_id = ClientOrderId(order_link_id) if order_link_id else None
@@ -343,7 +423,7 @@ class BybitExecutionClient(LiveExecutionClient):
             if client_order_id is None:
                 client_order_id = self._cache.client_order_id(venue_order_id)
 
-            order_report = targetOrder.parse_to_order_status_report(
+            order_report = target_order.parse_to_order_status_report(
                 client_order_id=client_order_id,
                 account_id=self.account_id,
                 instrument_id=instrument_id,
@@ -359,11 +439,10 @@ class BybitExecutionClient(LiveExecutionClient):
 
     async def generate_fill_reports(
         self,
-        instrument_id: InstrumentId | None = None,
-        venue_order_id: VenueOrderId | None = None,
-        start: pd.Timestamp | None = None,
-        end: pd.Timestamp | None = None,
+        command: GenerateFillReports,
     ) -> list[FillReport]:
+        instrument_id = command.instrument_id
+
         self._log.debug("Requesting FillReports...")
         reports: list[FillReport] = []
 
@@ -401,10 +480,10 @@ class BybitExecutionClient(LiveExecutionClient):
 
     async def generate_position_status_reports(
         self,
-        instrument_id: InstrumentId | None = None,
-        start: pd.Timestamp | None = None,
-        end: pd.Timestamp | None = None,
+        command: GeneratePositionStatusReports,
     ) -> list[PositionStatusReport]:
+        instrument_id = command.instrument_id
+
         reports: list[PositionStatusReport] = []
 
         try:
@@ -516,6 +595,72 @@ class BybitExecutionClient(LiveExecutionClient):
             except Exception as e:
                 self._log.error(f"Failed to generate AccountState: {e}")
 
+        # Set Leverages
+        if self._futures_leverages:
+            async with TaskGroup() as tg:
+                [
+                    tg.create_task(self.set_leverage(symbol=symbol, leverage=leverage))
+                    for symbol, leverage in self._futures_leverages.items()
+                    if symbol.is_linear or symbol.is_inverse
+                ]
+
+        # Set Position Mode
+        if self._position_mode:
+            async with TaskGroup() as tg:
+                [
+                    tg.create_task(self.set_position_mode(symbol=symbol, mode=mode))
+                    for symbol, mode in self._position_mode.items()
+                    if symbol.is_linear
+                ]
+
+        # Set Margin Mode
+        if self._margin_mode:
+            res_set_margin_mode = await self._http_account.set_margin_mode(self._margin_mode)
+            self._log.info(f"Set account margin mode result: {res_set_margin_mode.retMsg}")
+
+    async def set_leverage(
+        self,
+        symbol: BybitSymbol,
+        leverage: int,
+    ) -> None:
+        try:
+            res = await self._http_account.set_leverage(
+                category=symbol.product_type,
+                symbol=symbol.raw_symbol,
+                buy_leverage=str(leverage),
+                sell_leverage=str(leverage),
+            )
+            self._log.info(f"Set symbol `{symbol}` leverage to `{leverage}` result: {res.retMsg}")
+        except BybitError as e:
+            if e.code == 110043:  # Set leverage has not been modified. (already set)
+                self._log.info(
+                    f"Set symbol `{symbol}` leverage to `{leverage}` result: {e.message}",
+                )
+                return
+
+            raise e
+
+    async def set_position_mode(
+        self,
+        symbol: BybitSymbol,
+        mode: BybitPositionMode,
+    ) -> None:
+        try:
+            res = await self._http_account.switch_mode(
+                category=symbol.product_type,
+                symbol=symbol.raw_symbol,
+                mode=mode,
+            )
+            self._log.info(f"Set symbol `{symbol}` position mode to `{mode}` result: {res.retMsg}")
+        except BybitError as e:  # Position mode has not been modified. (already set)
+            if e.code == 110025:
+                self._log.info(
+                    f"Set symbol `{symbol}` position mode to `{mode}` result: {e.message}",
+                )
+                return
+
+            raise e
+
     # -- COMMAND HANDLERS -------------------------------------------------------------------------
 
     async def _cancel_order(self, command: CancelOrder) -> None:
@@ -526,8 +671,7 @@ class BybitExecutionClient(LiveExecutionClient):
 
         if order.is_closed:
             self._log.warning(
-                f"`CancelOrder` command for {command.client_order_id!r} when order already {order.status_string()} "
-                "(will not send to exchange)",
+                f"`CancelOrder` command for {command.client_order_id!r} when order already {order.status_string()} (will not send to exchange)",
             )
             return
 
@@ -539,7 +683,7 @@ class BybitExecutionClient(LiveExecutionClient):
             await retry_manager.run(
                 "cancel_order",
                 [client_order_id, venue_order_id],
-                self._http_account.cancel_order,
+                self._order_single_client.cancel_order,
                 bybit_symbol.product_type,
                 bybit_symbol.raw_symbol,
                 client_order_id=client_order_id,
@@ -593,7 +737,7 @@ class BybitExecutionClient(LiveExecutionClient):
                 await retry_manager.run(
                     "batch_cancel_orders",
                     None,
-                    self._http_account.batch_cancel_orders,
+                    self._order_batch_client.batch_cancel_orders,
                     product_type=product_type,
                     cancel_orders=cancel_orders,
                 )
@@ -644,8 +788,7 @@ class BybitExecutionClient(LiveExecutionClient):
 
         if order.is_closed:
             self._log.warning(
-                f"`ModifyOrder` command for {command.client_order_id!r} when order already {order.status_string()} "
-                "(will not send to exchange)",
+                f"`ModifyOrder` command for {command.client_order_id!r} when order already {order.status_string()} (will not send to exchange)",
             )
             return
 
@@ -660,7 +803,7 @@ class BybitExecutionClient(LiveExecutionClient):
             await retry_manager.run(
                 "modify_order",
                 [client_order_id, venue_order_id],
-                self._http_account.amend_order,
+                self._order_single_client.amend_order,
                 bybit_symbol.product_type,
                 bybit_symbol.raw_symbol,
                 client_order_id=client_order_id,
@@ -681,6 +824,7 @@ class BybitExecutionClient(LiveExecutionClient):
 
     async def _submit_order(self, command: SubmitOrder) -> None:
         order = command.order
+
         if order.is_closed:
             self._log.warning(f"Order {order} is already closed")
             return
@@ -700,7 +844,7 @@ class BybitExecutionClient(LiveExecutionClient):
         async with self._retry_manager_pool as retry_manager:
             await retry_manager.run(
                 "submit_order",
-                [command.order.client_order_id],
+                [order.client_order_id],
                 self._submit_order_methods[order.order_type],
                 order,
             )
@@ -716,10 +860,11 @@ class BybitExecutionClient(LiveExecutionClient):
     async def _submit_order_list(self, command: SubmitOrderList) -> None:
         bybit_symbol = BybitSymbol(command.instrument_id.symbol.value)
         product_type = bybit_symbol.product_type
+        command_orders = command.order_list.orders
         max_batch = 20 if product_type == BybitProductType.OPTION else 10
 
-        for i in range(0, len(command.order_list.orders), max_batch):
-            batch_submits = command.order_list.orders[i : i + max_batch]
+        for i in range(0, len(command_orders), max_batch):
+            batch_submits = command_orders[i : i + max_batch]
             submit_orders: list[BybitBatchPlaceOrder] = []
 
             for order in batch_submits:
@@ -727,21 +872,13 @@ class BybitExecutionClient(LiveExecutionClient):
                     self._log.error(f"Error on {command}")
                     return  # Do not submit batch
 
-                match order.order_type:
-                    case OrderType.MARKET:
-                        batch_order = self._create_market_batch_order(order)
-                    case OrderType.LIMIT:
-                        batch_order = self._create_limit_batch_order(order)
-                    case OrderType.LIMIT_IF_TOUCHED:
-                        batch_order = self._create_limit_if_touched_batch_order(order)
-                    case OrderType.STOP_MARKET:
-                        batch_order = self._create_stop_market_batch_order(order)
-                    case OrderType.MARKET_IF_TOUCHED:
-                        batch_order = self._create_market_if_touched_batch_order(order)
-                    case _:
-                        self._log.error(f"Unsupported order type for 'submit_order_list': {order}")
-                        self._log.error(f"Error on {command}")
-                        return
+                try:
+                    batch_order = self._batch_order_create_handlers[order.order_type](order)
+                except KeyError:
+                    self._log.error(
+                        f"Error on {command} - Unsupported order type for 'submit_order_list': {order}",
+                    )
+                    return
 
                 submit_orders.append(batch_order)
 
@@ -758,7 +895,7 @@ class BybitExecutionClient(LiveExecutionClient):
                 await retry_manager.run(
                     "submit_order_list",
                     None,
-                    self._http_account.batch_place_orders,
+                    self._order_batch_client.batch_place_orders,
                     product_type=product_type,
                     submit_orders=submit_orders,
                 )
@@ -784,7 +921,7 @@ class BybitExecutionClient(LiveExecutionClient):
         bybit_symbol = BybitSymbol(order.instrument_id.symbol.value)
         time_in_force = self._determine_time_in_force(order)
         order_side = self._enum_parser.parse_nautilus_order_side(order.side)
-        await self._http_account.place_order(
+        await self._order_single_client.place_order(
             product_type=bybit_symbol.product_type,
             symbol=bybit_symbol.raw_symbol,
             side=order_side,
@@ -800,7 +937,7 @@ class BybitExecutionClient(LiveExecutionClient):
         bybit_symbol = BybitSymbol(order.instrument_id.symbol.value)
         time_in_force = self._determine_time_in_force(order)
         order_side = self._enum_parser.parse_nautilus_order_side(order.side)
-        await self._http_account.place_order(
+        await self._order_single_client.place_order(
             product_type=bybit_symbol.product_type,
             symbol=bybit_symbol.raw_symbol,
             side=order_side,
@@ -820,7 +957,7 @@ class BybitExecutionClient(LiveExecutionClient):
         order_side = self._enum_parser.parse_nautilus_order_side(order.side)
         trigger_direction = self._enum_parser.parse_trigger_direction(order.order_type, order.side)
         trigger_type = self._enum_parser.parse_nautilus_trigger_type(order.trigger_type)
-        await self._http_account.place_order(
+        await self._order_single_client.place_order(
             product_type=product_type,
             symbol=bybit_symbol.raw_symbol,
             side=order_side,
@@ -845,7 +982,7 @@ class BybitExecutionClient(LiveExecutionClient):
         order_side = self._enum_parser.parse_nautilus_order_side(order.side)
         trigger_direction = self._enum_parser.parse_trigger_direction(order.order_type, order.side)
         trigger_type = self._enum_parser.parse_nautilus_trigger_type(order.trigger_type)
-        await self._http_account.place_order(
+        await self._order_single_client.place_order(
             product_type=product_type,
             symbol=bybit_symbol.raw_symbol,
             side=order_side,
@@ -870,7 +1007,7 @@ class BybitExecutionClient(LiveExecutionClient):
         order_type = BybitOrderType.MARKET
         trigger_direction = self._enum_parser.parse_trigger_direction(order.order_type, order.side)
         trigger_type = self._enum_parser.parse_nautilus_trigger_type(order.trigger_type)
-        await self._http_account.place_order(
+        await self._order_single_client.place_order(
             product_type=product_type,
             symbol=bybit_symbol.raw_symbol,
             side=order_side,
@@ -893,7 +1030,7 @@ class BybitExecutionClient(LiveExecutionClient):
         order_side = self._enum_parser.parse_nautilus_order_side(order.side)
         trigger_direction = self._enum_parser.parse_trigger_direction(order.order_type, order.side)
         trigger_type = self._enum_parser.parse_nautilus_trigger_type(order.trigger_type)
-        await self._http_account.place_order(
+        await self._order_single_client.place_order(
             product_type=product_type,
             symbol=bybit_symbol.raw_symbol,
             side=order_side,
@@ -926,7 +1063,10 @@ class BybitExecutionClient(LiveExecutionClient):
             trailing_offset=str(order.trailing_offset),
         )
 
-    def _handle_ws_message(self, raw: bytes) -> None:
+    def _handle_ws_message_trade(self, raw: bytes) -> None:
+        return
+
+    def _handle_ws_message_private(self, raw: bytes) -> None:
         try:
             ws_message = self._decoder_ws_msg_general.decode(raw)
             if ws_message.op == BYBIT_PONG:
@@ -934,17 +1074,24 @@ class BybitExecutionClient(LiveExecutionClient):
             if ws_message.success is False:
                 self._log.error(f"WebSocket error: {ws_message}")
                 return
-            if not ws_message.topic:
+
+            topic = ws_message.topic
+            if not topic:
                 return
 
-            if "order" in ws_message.topic:
+            # Sort by message frequency, from high to low for better performance
+            if "order" in topic:
                 self._handle_account_order_update(raw)
-            elif "execution" in ws_message.topic:
-                self._handle_account_execution_update(raw)
-            elif "wallet" == ws_message.topic:
+            # wallet has no `Categorised Topic`, `order` event should trigger `wallet` event
+            elif "wallet" == topic:
                 self._handle_account_wallet_update(raw)
+            elif "execution" in topic:
+                if "execution.fast" in topic:
+                    self._handle_account_execution_fast_update(raw)
+                else:
+                    self._handle_account_execution_update(raw)
             else:
-                self._log.error(f"Unknown websocket message topic: {ws_message.topic}")
+                self._log.error(f"Unknown websocket message topic: {topic}")
         except Exception as e:
             self._log.error(f"Failed to parse websocket message: {raw.decode()} with error {e}")
 
@@ -956,9 +1103,21 @@ class BybitExecutionClient(LiveExecutionClient):
         except Exception as e:
             self._log.exception(f"Failed to handle account execution update: {e}", e)
 
-    def _process_execution(self, execution: BybitWsAccountExecution) -> None:
+    def _handle_account_execution_fast_update(self, raw: bytes) -> None:
+        try:
+            msg = self._decoder_ws_account_execution_fast_update.decode(raw)
+            for trade in msg.data:
+                self._process_execution(trade)
+        except Exception as e:
+            self._log.exception(f"Failed to handle account execution update: {e}", e)
+
+    def _process_execution(
+        self,
+        execution: BybitWsAccountExecution | BybitWsAccountExecutionFast,
+    ) -> None:
         instrument_id = self._get_cached_instrument_id(execution.symbol, execution.category)
-        client_order_id = ClientOrderId(execution.orderLinkId) if execution.orderLinkId else None
+        order_link_id = execution.orderLinkId
+        client_order_id = ClientOrderId(order_link_id) if order_link_id else None
         venue_order_id = VenueOrderId(execution.orderId)
         order_side: OrderSide = self._enum_parser.parse_bybit_order_side(execution.side)
 
@@ -967,8 +1126,7 @@ class BybitExecutionClient(LiveExecutionClient):
 
         if client_order_id is None:
             self._log.debug(
-                f"Cannot process order execution for {venue_order_id!r}: no `ClientOrderId` found "
-                "(most likely due to being an external order)",
+                f"Cannot process order execution for {venue_order_id!r}: no `ClientOrderId` found (most likely due to being an external order)",
             )
             return
 
@@ -997,6 +1155,15 @@ class BybitExecutionClient(LiveExecutionClient):
         if instrument is None:
             raise ValueError(f"Cannot handle trade event: instrument {instrument_id} not found")
 
+        quote_currency = instrument.quote_currency
+        is_maker = execution.isMaker
+        fee = instrument.maker_fee if is_maker else instrument.taker_fee
+
+        last_qty: Quantity = instrument.make_qty(execution.execQty)
+        last_px: Price = instrument.make_price(execution.execPrice)
+        notional_value: Money = instrument.notional_value(last_qty, last_px)
+        commission: Money = Money(notional_value * fee, quote_currency)
+
         self.generate_order_filled(
             strategy_id=strategy_id,
             instrument_id=instrument_id,
@@ -1006,11 +1173,11 @@ class BybitExecutionClient(LiveExecutionClient):
             trade_id=TradeId(execution.execId),
             order_side=order_side,
             order_type=order_type,
-            last_qty=Quantity(float(execution.execQty), instrument.size_precision),
-            last_px=Price(float(execution.execPrice), instrument.price_precision),
-            quote_currency=instrument.quote_currency,
-            commission=Money(Decimal(execution.execFee), instrument.quote_currency),
-            liquidity_side=LiquiditySide.MAKER if execution.isMaker else LiquiditySide.TAKER,
+            last_qty=last_qty,
+            last_px=last_px,
+            quote_currency=quote_currency,
+            commission=commission,
+            liquidity_side=LiquiditySide.MAKER if is_maker else LiquiditySide.TAKER,
             ts_event=millis_to_nanos(float(execution.execTime)),
         )
 
@@ -1119,9 +1286,9 @@ class BybitExecutionClient(LiveExecutionClient):
                             venue_order_id=report.venue_order_id,
                             ts_event=report.ts_last,
                         )
-                elif (
-                    bybit_order.orderStatus == BybitOrderStatus.CANCELED
-                    or bybit_order.orderStatus == BybitOrderStatus.DEACTIVATED
+                elif bybit_order.orderStatus in (
+                    BybitOrderStatus.CANCELED,
+                    BybitOrderStatus.DEACTIVATED,
                 ):
                     self.generate_order_canceled(
                         strategy_id=strategy_id,
