@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -32,7 +32,14 @@ from nautilus_trader.adapters.bybit.common.enums import BybitStopOrderType
 from nautilus_trader.adapters.bybit.common.enums import BybitTimeInForce
 from nautilus_trader.adapters.bybit.common.enums import BybitTriggerDirection
 from nautilus_trader.adapters.bybit.common.enums import BybitTriggerType
+from nautilus_trader.adapters.bybit.common.enums import BybitWsOrderRequestMsgOP
 from nautilus_trader.adapters.bybit.common.parsing import parse_bybit_delta
+from nautilus_trader.adapters.bybit.endpoints.trade.amend_order import BybitAmendOrderPostParams
+from nautilus_trader.adapters.bybit.endpoints.trade.cancel_order import BybitCancelOrderPostParams
+from nautilus_trader.adapters.bybit.endpoints.trade.place_order import BybitPlaceOrderPostParams
+from nautilus_trader.adapters.bybit.schemas.order import BybitAmendOrder
+from nautilus_trader.adapters.bybit.schemas.order import BybitCancelOrder
+from nautilus_trader.adapters.bybit.schemas.order import BybitPlaceOrder
 from nautilus_trader.core.datetime import millis_to_nanos
 from nautilus_trader.core.uuid import UUID4
 from nautilus_trader.execution.reports import OrderStatusReport
@@ -79,6 +86,27 @@ class BybitWsSubscriptionMsg(msgspec.Struct):
     req_id: str | None = None
 
 
+class BybitWsPrivateChannelAuthMsg(msgspec.Struct, kw_only=True):
+    success: bool
+    ret_msg: str | None = None
+    op: str
+    conn_id: str
+
+    def is_auth_success(self) -> bool:
+        return (self.op == "auth") and (self.success is True)
+
+
+class BybitWsTradeAuthMsg(msgspec.Struct, kw_only=True):
+    reqId: str | None = None
+    retCode: int
+    retMsg: str
+    op: str
+    connId: str
+
+    def is_auth_success(self) -> bool:
+        return (self.op == "auth") and (self.retCode == 0)
+
+
 ################################################################################
 # Public - Kline
 ################################################################################
@@ -184,18 +212,19 @@ class BybitWsOrderbookDepth(msgspec.Struct):
             for d in self.a
         ]
 
-        num_bids_raw = len(bids_raw)
-        num_asks_raw = len(asks_raw)
         deltas: list[OrderBookDelta] = []
 
         if snapshot:
             deltas.append(OrderBookDelta.clear(instrument_id, 0, ts_event, ts_init))
 
-        for bid_id, bid in enumerate(bids_raw):
+        bids_len = len(bids_raw)
+        asks_len = len(asks_raw)
+
+        for idx, bid in enumerate(bids_raw):
             flags = 0
-            if bid_id == num_bids_raw - 1 and num_asks_raw == 0:
+            if idx == bids_len - 1 and asks_len == 0:
                 # F_LAST, 1 << 7
-                # Last message in the packet from the venue for a given `instrument_id`
+                # Last message in the book event or packet from the venue for a given `instrument_id`
                 flags = RecordFlag.F_LAST
 
             delta = parse_bybit_delta(
@@ -211,11 +240,11 @@ class BybitWsOrderbookDepth(msgspec.Struct):
             )
             deltas.append(delta)
 
-        for ask_id, ask in enumerate(asks_raw):
+        for idx, ask in enumerate(asks_raw):
             flags = 0
-            if ask_id == num_asks_raw - 1:
+            if idx == asks_len - 1:
                 # F_LAST, 1 << 7
-                # Last message in the packet from the venue for a given `instrument_id`
+                # Last message in the book event or packet from the venue for a given `instrument_id`
                 flags = RecordFlag.F_LAST
 
             delta = parse_bybit_delta(
@@ -514,16 +543,21 @@ class BybitWsAccountPosition(msgspec.Struct):
     takeProfit: str
     stopLoss: str
     trailingStop: str
+    sessionAvgPrice: str
     unrealisedPnl: str
     cumRealisedPnl: str
     createdTime: str
     updatedTime: str
     liqPrice: str
     bustPrice: str
-    category: str
+    category: BybitProductType
     positionStatus: str
     adlRankIndicator: int
     seq: int
+    autoAddMargin: int
+    leverageSysUpdatedTime: str
+    mmrSysUpdatedTime: str
+    isReduceOnly: bool
     tpslMode: str | None = None
 
 
@@ -686,6 +720,34 @@ class BybitWsAccountExecutionMsg(msgspec.Struct):
     data: list[BybitWsAccountExecution]
 
 
+################################################################################
+# Private - Account Execution Fast
+################################################################################
+
+
+class BybitWsAccountExecutionFast(msgspec.Struct):
+    category: BybitProductType
+    symbol: str
+    orderId: str
+    isMaker: bool
+    orderLinkId: str
+    side: BybitOrderSide
+    execId: str
+    execPrice: str
+    execQty: str
+    execTime: str
+    seq: int
+
+    orderType: BybitOrderType = BybitOrderType.UNKNOWN
+    stopOrderType: BybitStopOrderType = BybitStopOrderType.NONE
+
+
+class BybitWsAccountExecutionFastMsg(msgspec.Struct):
+    topic: str
+    creationTime: int
+    data: list[BybitWsAccountExecutionFast]
+
+
 ################################################################################
 # Private - Account Wallet
 ################################################################################
@@ -765,3 +827,38 @@ class BybitWsAccountWalletMsg(msgspec.Struct):
                 reported=True,
                 ts_event=millis_to_nanos(self.creationTime),
             )
+
+
+################################################################################
+# Trade
+################################################################################
+
+
+class BybitWsOrderRequestMsg(msgspec.Struct, kw_only=True):
+    reqId: str | None = None
+    header: dict[str, str]
+    op: BybitWsOrderRequestMsgOP
+    args: list[
+        BybitPlaceOrderPostParams | BybitAmendOrderPostParams | BybitCancelOrderPostParams
+    ] = []
+
+
+class BybitWsOrderResponseMsg(msgspec.Struct, kw_only=True):
+    reqId: str | None = None
+    retCode: int
+    retMsg: str
+    op: str
+    header: dict[str, str] | None = None
+    connId: str
+
+
+class BybitWsPlaceOrderResponseMsg(BybitWsOrderResponseMsg):
+    data: BybitPlaceOrder
+
+
+class BybitWsAmendOrderResponseMsg(BybitWsOrderResponseMsg):
+    data: BybitAmendOrder
+
+
+class BybitWsCancelOrderResponseMsg(BybitWsOrderResponseMsg):
+    data: BybitCancelOrder
