@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -20,6 +20,7 @@ from typing import Any
 import pandas as pd
 import pytz
 
+from nautilus_trader.adapters.databento.common import instrument_id_to_pyo3
 from nautilus_trader.adapters.databento.constants import ALL_SYMBOLS
 from nautilus_trader.adapters.databento.constants import PUBLISHERS_FILEPATH
 from nautilus_trader.adapters.databento.enums import DatabentoSchema
@@ -125,6 +126,7 @@ class DatabentoInstrumentProvider(InstrumentProvider):
         )
 
         parent_symbols = list(filters.get("parent_symbols", [])) if filters is not None else None
+        # use_exchange_as_venue = filters.get("use_exchange_as_venue", True) if filters else True
 
         pyo3_instruments = []
 
@@ -152,15 +154,21 @@ class DatabentoInstrumentProvider(InstrumentProvider):
 
         live_client.subscribe(
             schema=DatabentoSchema.DEFINITION.value,
-            symbols=sorted([i.symbol.value for i in instrument_ids]),
+            instrument_ids=sorted(  # type: ignore[type-var]
+                [instrument_id_to_pyo3(instrument_id) for instrument_id in instrument_ids],
+            ),
             start=0,  # From start of current week (latest definitions)
+            # use_exchange_as_venue=use_exchange_as_venue,  # TODO
         )
 
         if parent_symbols:
             self._log.info(f"Requesting parent symbols {parent_symbols}", LogColor.BLUE)
             live_client.subscribe(
                 schema=DatabentoSchema.DEFINITION.value,
-                symbols=parent_symbols,
+                instrument_ids=[
+                    instrument_id_to_pyo3(InstrumentId.from_str(f"{parent_symbol}.GLBX"))
+                    for parent_symbol in parent_symbols
+                ],
                 start=0,  # From start of current week (latest definitions)
             )
 
@@ -220,7 +228,7 @@ class DatabentoInstrumentProvider(InstrumentProvider):
         Calling this method will incur a cost to your Databento account in USD.
 
         """
-        await self.load_ids_async([instrument_id])
+        await self.load_ids_async([instrument_id], filters=filters)
 
     async def get_range(
         self,
@@ -256,12 +264,16 @@ class DatabentoInstrumentProvider(InstrumentProvider):
 
         """
         dataset = self._check_all_datasets_equal(instrument_ids)
+        use_exchange_as_venue = filters.get("use_exchange_as_venue", True) if filters else True
 
+        # Here the NULL venue is overridden and so is used as a
+        # placeholder to conform to instrument ID conventions.
         pyo3_instruments = await self._http_client.get_range_instruments(
             dataset=dataset,
-            symbols=[ALL_SYMBOLS],
+            instrument_ids=[instrument_id_to_pyo3(InstrumentId.from_str(f"{ALL_SYMBOLS}.NULL"))],
             start=pd.Timestamp(start, tz=pytz.utc).value,
             end=pd.Timestamp(end, tz=pytz.utc).value if end is not None else None,
+            use_exchange_as_venue=use_exchange_as_venue,
         )
 
         instruments = instruments_from_pyo3(pyo3_instruments)
