@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -53,6 +53,8 @@ from nautilus_trader.adapters.dydx.providers import DYDXInstrumentProvider
 from nautilus_trader.adapters.dydx.schemas.ws import DYDXWsBlockHeightChannelData
 from nautilus_trader.adapters.dydx.schemas.ws import DYDXWsBlockHeightSubscribedData
 from nautilus_trader.adapters.dydx.schemas.ws import DYDXWsFillSubaccountMessageContents
+from nautilus_trader.adapters.dydx.schemas.ws import DYDXWsMarketChannelData
+from nautilus_trader.adapters.dydx.schemas.ws import DYDXWsMarketSubscribedData
 from nautilus_trader.adapters.dydx.schemas.ws import DYDXWsMessageGeneral
 from nautilus_trader.adapters.dydx.schemas.ws import DYDXWsOrderSubaccountMessageContents
 from nautilus_trader.adapters.dydx.schemas.ws import DYDXWsSubaccountsChannelData
@@ -60,8 +62,10 @@ from nautilus_trader.adapters.dydx.schemas.ws import DYDXWsSubaccountsSubscribed
 from nautilus_trader.adapters.dydx.websocket.client import DYDXWebsocketClient
 from nautilus_trader.cache.cache import Cache
 from nautilus_trader.common.component import LiveClock
+from nautilus_trader.common.component import Logger
 from nautilus_trader.common.component import MessageBus
 from nautilus_trader.common.enums import LogColor
+from nautilus_trader.common.enums import LogLevel
 from nautilus_trader.core.correctness import PyCondition
 from nautilus_trader.core.datetime import dt_to_unix_nanos
 from nautilus_trader.core.datetime import nanos_to_secs
@@ -69,6 +73,10 @@ from nautilus_trader.core.uuid import UUID4
 from nautilus_trader.execution.messages import BatchCancelOrders
 from nautilus_trader.execution.messages import CancelAllOrders
 from nautilus_trader.execution.messages import CancelOrder
+from nautilus_trader.execution.messages import GenerateFillReports
+from nautilus_trader.execution.messages import GenerateOrderStatusReport
+from nautilus_trader.execution.messages import GenerateOrderStatusReports
+from nautilus_trader.execution.messages import GeneratePositionStatusReports
 from nautilus_trader.execution.messages import SubmitOrder
 from nautilus_trader.execution.messages import SubmitOrderList
 from nautilus_trader.execution.reports import FillReport
@@ -110,6 +118,7 @@ class ClientOrderIdHelper:
         Generate integer client order IDs.
         """
         self._cache = cache
+        self._log: Logger = Logger(type(self).__name__)
 
     def generate_client_order_id_int(self, client_order_id: ClientOrderId) -> int:
         """
@@ -143,6 +152,8 @@ class ClientOrderIdHelper:
 
             if value is not None:
                 result = int.from_bytes(value, byteorder="big")
+            else:
+                self._log.error(f"ClientOrderId integer not found in cache for {client_order_id!r}")
 
         return result
 
@@ -154,6 +165,8 @@ class ClientOrderIdHelper:
 
         if value is not None:
             return ClientOrderId(value.decode("utf-8"))
+        else:
+            self._log.error(f"ClientOrderId not found in cache for integer {client_order_id_int}")
 
         return ClientOrderId(str(client_order_id_int))
 
@@ -228,6 +241,7 @@ class DYDXExecutionClient(LiveExecutionClient):
             f"{name or DYDX_VENUE.value}-{self._wallet_address}-{self._subaccount}",
         )
         self._set_account_id(account_id)
+        self._connect_account_timeout_secs = 10
 
         # WebSocket API
         self._ws_client = DYDXWebsocketClient(
@@ -263,13 +277,16 @@ class DYDXExecutionClient(LiveExecutionClient):
             DYDXWsBlockHeightSubscribedData,
         )
         self._decoder_ws_block_height_channel = msgspec.json.Decoder(DYDXWsBlockHeightChannelData)
+        self._decoder_ws_instruments = msgspec.json.Decoder(DYDXWsMarketChannelData)
+        self._decoder_ws_instruments_subscribed = msgspec.json.Decoder(DYDXWsMarketSubscribedData)
 
         # Hot caches
         self._order_builders: dict[InstrumentId, OrderBuilder] = {}
         self._generate_order_status_retries: dict[ClientOrderId, int] = {}
         self._block_height: int = 0
+        self._oracle_prices: dict[InstrumentId, Decimal] = {}
 
-        self._retry_manager_pool = RetryManagerPool(
+        self._retry_manager_pool = RetryManagerPool[None](
             pool_size=100,
             max_retries=config.max_retries or 0,
             retry_delay_secs=config.retry_delay or 1.0,
@@ -288,11 +305,12 @@ class DYDXExecutionClient(LiveExecutionClient):
         await self._ws_client.connect()
 
         # Subscribe account updates
+        await self._ws_client.subscribe_markets()
+        await self._ws_client.subscribe_block_height()
         await self._ws_client.subscribe_account_update(
             wallet_address=self._wallet_address,
             subaccount_number=self._subaccount,
         )
-        await self._ws_client.subscribe_block_height()
 
         self._block_height = await self._grpc_account.latest_block_height()
 
@@ -303,12 +321,34 @@ class DYDXExecutionClient(LiveExecutionClient):
             sequence=account.sequence,
         )
 
+        await self._set_leverage()
+
+    async def _set_leverage(self) -> None:
+        timeout = self._clock.utc_now() + pd.Timedelta(seconds=self._connect_account_timeout_secs)
+        account = self.get_account()
+
+        while account is None and self._clock.utc_now() < timeout:
+            await asyncio.sleep(0.1)
+            account = self.get_account()
+
+        if account is None:
+            self._log.error("Account is not initialized")
+            return
+
+        instruments = self._instrument_provider.get_all()
+
+        for instrument_id, instrument in instruments.items():
+            leverage = Decimal(1) / instrument.margin_init
+            account.set_leverage(instrument_id, leverage)
+
     async def _disconnect(self) -> None:
+        await self._ws_client.unsubscribe_markets()
+        await self._ws_client.unsubscribe_block_height()
         await self._ws_client.unsubscribe_account_update(
             wallet_address=self._wallet_address,
             subaccount_number=self._subaccount,
         )
-        await self._ws_client.unsubscribe_block_height()
+
         await self._ws_client.disconnect()
         await self._grpc_account.disconnect()
 
@@ -385,39 +425,37 @@ class DYDXExecutionClient(LiveExecutionClient):
 
     async def generate_order_status_report(  # noqa: C901
         self,
-        instrument_id: InstrumentId,
-        client_order_id: ClientOrderId | None = None,
-        venue_order_id: VenueOrderId | None = None,
+        command: GenerateOrderStatusReport,
     ) -> OrderStatusReport | None:
         """
         Create an order status report for a specific order.
         """
         self._log.debug("Requesting OrderStatusReport...")
         PyCondition.is_false(
-            client_order_id is None and venue_order_id is None,
+            command.client_order_id is None and command.venue_order_id is None,
             "both `client_order_id` and `venue_order_id` were `None`",
         )
 
         max_retries = 3
-        retries = self._generate_order_status_retries.get(client_order_id, 0)
+        retries = self._generate_order_status_retries.get(command.client_order_id, 0)
 
         if retries > max_retries:
             self._log.error(
                 f"Reached maximum retries {retries}/{max_retries} for generating OrderStatusReport for "
-                f"{repr(client_order_id) if client_order_id else ''} "
-                f"{repr(venue_order_id) if venue_order_id else ''}",
+                f"{repr(command.client_order_id) if command.client_order_id else ''} "
+                f"{repr(command.venue_order_id) if command.venue_order_id else ''}",
             )
             return None
 
         self._log.info(
-            f"Generating OrderStatusReport for {repr(client_order_id) if client_order_id else ''} {repr(venue_order_id) if venue_order_id else ''}",
+            f"Generating OrderStatusReport for {repr(command.client_order_id) if command.client_order_id else ''} {repr(command.venue_order_id) if command.venue_order_id else ''}",  # noqa: E501
         )
 
         report = None
         order = None
 
-        if client_order_id is None:
-            client_order_id = self._cache.client_order_id(venue_order_id)
+        if command.client_order_id is None:
+            client_order_id = self._cache.client_order_id(command.venue_order_id)
 
         if client_order_id:
             order = self._cache.order(client_order_id)
@@ -430,12 +468,12 @@ class DYDXExecutionClient(LiveExecutionClient):
         if order.is_closed:
             return None  # Nothing else to do
 
-        if venue_order_id is None:
+        if command.venue_order_id is None:
             venue_order_id = order.venue_order_id
 
         try:
             report = await self._get_order_status_report(
-                instrument_id=instrument_id,
+                instrument_id=command.instrument_id,
                 client_order_id=client_order_id,
                 venue_order_id=venue_order_id,
                 order_side=order.side,
@@ -457,7 +495,7 @@ class DYDXExecutionClient(LiveExecutionClient):
                 # so that there are no longer subsequent retries (we don't expect many of these).
                 self.generate_order_rejected(
                     strategy_id=order.strategy_id,
-                    instrument_id=instrument_id,
+                    instrument_id=command.instrument_id,
                     client_order_id=client_order_id,
                     reason=str(e.message),
                     ts_event=self._clock.timestamp_ns(),
@@ -473,10 +511,7 @@ class DYDXExecutionClient(LiveExecutionClient):
 
     async def generate_order_status_reports(
         self,
-        instrument_id: InstrumentId | None = None,
-        start: pd.Timestamp | None = None,
-        end: pd.Timestamp | None = None,
-        open_only: bool = False,
+        command: GenerateOrderStatusReports,
     ) -> list[OrderStatusReport]:
         """
         Create an order status report.
@@ -485,18 +520,20 @@ class DYDXExecutionClient(LiveExecutionClient):
         reports: list[OrderStatusReport] = []
 
         symbol = None
-        start_dt = start.to_pydatetime() if start is not None else None
-        end_dt = end.to_pydatetime() if end is not None else None
+        start_dt = command.start.to_pydatetime() if command.start is not None else None
+        end_dt = command.end.to_pydatetime() if command.end is not None else None
 
-        if instrument_id is not None:
-            symbol = instrument_id.symbol.value.removesuffix("-PERP")
+        if command.instrument_id is not None:
+            symbol = command.instrument_id.symbol.value.removesuffix("-PERP")
 
         dydx_orders = await self._http_account.get_orders(
             address=self._wallet_address,
             subaccount_number=self._subaccount,
             symbol=symbol,
             order_status=(
-                [DYDXOrderStatus.OPEN, DYDXOrderStatus.BEST_EFFORT_OPENED] if open_only else None
+                [DYDXOrderStatus.OPEN, DYDXOrderStatus.BEST_EFFORT_OPENED]
+                if command.open_only
+                else None
             ),
         )
 
@@ -545,16 +582,18 @@ class DYDXExecutionClient(LiveExecutionClient):
 
         len_reports = len(reports)
         plural = "" if len_reports == 1 else "s"
-        self._log.info(f"Received {len(reports)} OrderStatusReport{plural}")
+        receipt_log = f"Received {len(reports)} OrderStatusReport{plural}"
+
+        if command.log_receipt_level == LogLevel.INFO:
+            self._log.info(receipt_log)
+        else:
+            self._log.debug(receipt_log)
 
         return reports
 
     async def generate_fill_reports(
         self,
-        instrument_id: InstrumentId | None = None,
-        venue_order_id: VenueOrderId | None = None,
-        start: pd.Timestamp | None = None,
-        end: pd.Timestamp | None = None,
+        command: GenerateFillReports,
     ) -> list[FillReport]:
         """
         Create an order fill report.
@@ -563,11 +602,11 @@ class DYDXExecutionClient(LiveExecutionClient):
         reports: list[FillReport] = []
 
         symbol = None
-        start_dt = start.to_pydatetime() if start is not None else None
-        end_dt = end.to_pydatetime() if end is not None else None
+        start_dt = command.start.to_pydatetime() if command.start is not None else None
+        end_dt = command.end.to_pydatetime() if command.end is not None else None
 
-        if instrument_id is not None:
-            symbol = instrument_id.symbol.value.removesuffix("-PERP")
+        if command.instrument_id is not None:
+            symbol = command.instrument_id.symbol.value.removesuffix("-PERP")
 
         dydx_fills = await self._http_account.get_fills(
             address=self._wallet_address,
@@ -623,9 +662,7 @@ class DYDXExecutionClient(LiveExecutionClient):
 
     async def generate_position_status_reports(
         self,
-        instrument_id: InstrumentId | None = None,
-        start: pd.Timestamp | None = None,
-        end: pd.Timestamp | None = None,
+        command: GeneratePositionStatusReports,
     ) -> list[PositionStatusReport]:
         """
         Generate position status reports.
@@ -640,11 +677,11 @@ class DYDXExecutionClient(LiveExecutionClient):
         )
 
         if dydx_positions is not None:
-            if instrument_id:
+            if command.instrument_id:
                 for dydx_position in dydx_positions.positions:
                     current_instrument_id = DYDXSymbol(dydx_position.market).to_instrument_id()
 
-                    if current_instrument_id == instrument_id:
+                    if current_instrument_id == command.instrument_id:
                         instrument = self._cache.instrument(current_instrument_id)
 
                         if instrument is None:
@@ -666,7 +703,7 @@ class DYDXExecutionClient(LiveExecutionClient):
                     now = self._clock.timestamp_ns()
                     report = PositionStatusReport(
                         account_id=self.account_id,
-                        instrument_id=instrument_id,
+                        instrument_id=command.instrument_id,
                         position_side=PositionSide.FLAT,
                         quantity=Quantity.zero(),
                         report_id=UUID4(),
@@ -702,26 +739,38 @@ class DYDXExecutionClient(LiveExecutionClient):
         self._log.info(f"Received {len(reports)} PositionStatusReport{plural}")
         return reports
 
-    def _handle_ws_message(self, raw: bytes) -> None:
+    def _handle_ws_message(self, raw: bytes) -> None:  # noqa: C901
         try:
             ws_message = self._decoder_ws_msg_general.decode(raw)
+            ws_message_channel = ws_message.channel
+            ws_message_type = ws_message.type
 
-            if ws_message.channel == "v4_block_height" and ws_message.type == "channel_data":
-                self._handle_block_height_channel_data(raw)
-            elif ws_message.channel == "v4_subaccounts" and ws_message.type == "channel_data":
-                self._handle_subaccounts_channel_data(raw)
-            elif ws_message.channel == "v4_block_height" and ws_message.type == "subscribed":
-                self._handle_block_height_subscribed(raw)
-            elif ws_message.channel == "v4_subaccounts" and ws_message.type == "subscribed":
-                self._handle_subaccounts_subscribed(raw)
-            elif ws_message.type == "unsubscribed":
+            if ws_message_type == "channel_data":
+                if ws_message_channel == "v4_block_height":
+                    self._handle_block_height_channel_data(raw)
+                elif ws_message_channel == "v4_subaccounts":
+                    self._handle_subaccounts_channel_data(raw)
+                elif ws_message_channel == "v4_markets":
+                    self._handle_markets(raw)
+                else:
+                    self._log.error(f"Unknown message `{ws_message_type}`: {raw.decode()}")
+            elif ws_message_type == "subscribed":
+                if ws_message_channel == "v4_block_height":
+                    self._handle_block_height_subscribed(raw)
+                elif ws_message_channel == "v4_subaccounts":
+                    self._handle_subaccounts_subscribed(raw)
+                elif ws_message_channel == "v4_markets":
+                    self._handle_markets_subscribed(raw)
+                else:
+                    self._log.error(f"Unknown message `{ws_message_type}`: {raw.decode()}")
+            elif ws_message_type == "unsubscribed":
                 self._log.info(
-                    f"Unsubscribed from channel {ws_message.channel} for {ws_message.id}",
+                    f"Unsubscribed from channel {ws_message_channel} for {ws_message.id}",
                 )
-            elif ws_message.type == "connected":
+            elif ws_message_type == "connected":
                 self._log.info("Websocket connected")
             else:
-                self._log.error(f"Unknown message `{ws_message.type}`: {raw.decode()}")
+                self._log.error(f"Unknown message `{ws_message_type}`: {raw.decode()}")
         except Exception as e:
             self._log.error(f"Failed to parse websocket message: {raw.decode()} with error {e}")
 
@@ -749,6 +798,30 @@ class DYDXExecutionClient(LiveExecutionClient):
                 f"Failed to parse block height channel message: {raw.decode()} with error {e}",
             )
 
+    def _handle_markets(self, raw: bytes) -> None:
+        try:
+            msg: DYDXWsMarketChannelData = self._decoder_ws_instruments.decode(raw)
+
+            if msg.contents.oraclePrices is not None:
+                for symbol, oracle_price_market in msg.contents.oraclePrices.items():
+                    instrument_id = DYDXSymbol(symbol).to_instrument_id()
+                    self._oracle_prices[instrument_id] = Decimal(oracle_price_market.oraclePrice)
+
+        except Exception as e:
+            self._log.error(f"Failed to parse market data: {raw.decode()} with error {e}")
+
+    def _handle_markets_subscribed(self, raw: bytes) -> None:
+        try:
+            msg: DYDXWsMarketSubscribedData = self._decoder_ws_instruments_subscribed.decode(raw)
+
+            for symbol, oracle_price_market in msg.contents.markets.items():
+                if oracle_price_market.oraclePrice is not None:
+                    instrument_id = DYDXSymbol(symbol).to_instrument_id()
+                    self._oracle_prices[instrument_id] = Decimal(oracle_price_market.oraclePrice)
+
+        except Exception as e:
+            self._log.error(f"Failed to parse market channel data: {raw.decode()} with error {e}")
+
     def _handle_subaccounts_subscribed(self, raw: bytes) -> None:
         try:
             msg: DYDXWsSubaccountsSubscribed = self._decoder_ws_msg_subaccounts_subscribed.decode(
@@ -780,6 +853,7 @@ class DYDXExecutionClient(LiveExecutionClient):
                 margin_balance = perpetual_position.parse_margin_balance(
                     margin_init=instrument.margin_init,
                     margin_maint=instrument.margin_maint,
+                    oracle_price=self._oracle_prices.get(instrument.id),
                 )
 
                 initial_margins[
@@ -896,7 +970,7 @@ class DYDXExecutionClient(LiveExecutionClient):
             # Skip order filled message and best effort canceled message. The _handle_fill_message generates
             # a fill report.
             # Best effort canceled is not a terminal state. Hence, we keep the state at accepted.
-            self._log.debug(f"Skip order message: {order_msg}")
+            self._log.info(f"Skip order message: {order_msg}")
         else:
             message = f"Unknown order status `{order_msg.status}`"
             self._log.error(message)
@@ -1145,7 +1219,7 @@ class DYDXExecutionClient(LiveExecutionClient):
 
         await self._place_order(order_msg=order_msg, order=order)
 
-    async def _place_order(self, order_msg: DYDXOrder, order: Order):
+    async def _place_order(self, order_msg: DYDXOrder, order: Order) -> None:
         if self._wallet is None:
             rejection_reason = "Cannot submit order: no wallet available"
             self._log.error(rejection_reason)
