@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -23,6 +23,7 @@ from decimal import Decimal
 
 import msgspec
 
+from nautilus_trader.adapters.dydx.common.constants import DEFAULT_CURRENCY
 from nautilus_trader.adapters.dydx.common.enums import DYDXEnumParser
 from nautilus_trader.adapters.dydx.common.enums import DYDXFillType
 from nautilus_trader.adapters.dydx.common.enums import DYDXLiquidity
@@ -61,6 +62,8 @@ from nautilus_trader.model.identifiers import InstrumentId
 from nautilus_trader.model.identifiers import TradeId
 from nautilus_trader.model.identifiers import VenueOrderId
 from nautilus_trader.model.objects import AccountBalance
+from nautilus_trader.model.objects import Currency
+from nautilus_trader.model.objects import Money
 from nautilus_trader.model.objects import Price
 from nautilus_trader.model.objects import Quantity
 
@@ -98,15 +101,14 @@ class DYDXCandle(msgspec.Struct, forbid_unknown_fields=True):
         high_price = Price(Decimal(self.high), price_precision)
         low_price = Price(Decimal(self.low), price_precision)
         close_price = Price(Decimal(self.close), price_precision)
-        avg_price = Decimal("0.25") * (open_price + high_price + low_price + close_price)
-        volume = Decimal(self.usdVolume) / avg_price
+        volume = Quantity(Decimal(self.baseTokenVolume), size_precision)
         return Bar(
             bar_type=bar_type,
             open=open_price,
             high=high_price,
             low=low_price,
             close=close_price,
-            volume=Quantity(volume, size_precision),
+            volume=volume,
             ts_event=dt_to_unix_nanos(self.startedAt),
             ts_init=ts_init,
         )
@@ -262,16 +264,16 @@ class DYDXWsOrderbookChannelData(msgspec.Struct, forbid_unknown_fields=True):
         if self.contents.asks is None:
             self.contents.asks = []
 
-        num_bids_raw = len(self.contents.bids)
-        num_asks_raw = len(self.contents.asks)
         deltas: list[OrderBookDelta] = []
 
-        for bid_id, bid in enumerate(self.contents.bids):
-            flags = 0
+        bids_len = len(self.contents.bids)
+        asks_len = len(self.contents.asks)
 
-            if bid_id == num_bids_raw - 1 and num_asks_raw == 0:
+        for idx, bid in enumerate(self.contents.bids):
+            flags = 0
+            if idx == bids_len - 1 and asks_len == 0:
                 # F_LAST, 1 << 7
-                # Last message in the packet from the venue for a given `instrument_id`
+                # Last message in the book event or packet from the venue for a given `instrument_id`
                 flags = RecordFlag.F_LAST
 
             size = Quantity(Decimal(bid[1]), size_precision)
@@ -292,10 +294,9 @@ class DYDXWsOrderbookChannelData(msgspec.Struct, forbid_unknown_fields=True):
             )
             deltas.append(delta)
 
-        for ask_id, ask in enumerate(self.contents.asks):
+        for idx, ask in enumerate(self.contents.asks):
             flags = 0
-
-            if ask_id == num_asks_raw - 1:
+            if idx == asks_len - 1:
                 # F_LAST, 1 << 7
                 # Last message in the book event or packet from the venue for a given `instrument_id`
                 flags = RecordFlag.F_LAST
@@ -338,6 +339,88 @@ class DYDXWsOrderbookMessageSnapshotContents(msgspec.Struct, forbid_unknown_fiel
     bids: list[PriceLevel] | None = None
     asks: list[PriceLevel] | None = None
 
+    def parse_to_snapshot(
+        self,
+        instrument_id: InstrumentId,
+        price_precision: int,
+        size_precision: int,
+        ts_event: int,
+        ts_init: int,
+    ) -> OrderBookDeltas:
+        """
+        Parse the order book message into OrderBookDeltas.
+        """
+        deltas: list[OrderBookDelta] = []
+
+        # Add initial clear
+        clear = OrderBookDelta.clear(
+            instrument_id=instrument_id,
+            sequence=0,
+            ts_event=ts_event,
+            ts_init=ts_init,
+        )
+        deltas.append(clear)
+
+        if self.bids is None:
+            self.bids = []
+
+        if self.asks is None:
+            self.asks = []
+
+        bids_len = len(self.bids)
+        asks_len = len(self.asks)
+
+        for idx, bid in enumerate(self.bids):
+            flags = 0
+            if idx == bids_len - 1 and asks_len == 0:
+                # F_LAST, 1 << 7
+                # Last message in the book event or packet from the venue for a given `instrument_id`
+                flags = RecordFlag.F_LAST
+
+            order = BookOrder(
+                side=OrderSide.BUY,
+                price=Price(Decimal(bid.price), price_precision),
+                size=Quantity(Decimal(bid.size), size_precision),
+                order_id=0,
+            )
+
+            delta = OrderBookDelta(
+                instrument_id=instrument_id,
+                action=BookAction.ADD,
+                order=order,
+                flags=flags,
+                sequence=0,
+                ts_event=ts_event,
+                ts_init=ts_init,
+            )
+
+            deltas.append(delta)
+
+        for idx, ask in enumerate(self.asks):
+            flags = 0
+            if idx == asks_len - 1:
+                # F_LAST, 1 << 7
+                # Last message in the book event or packet from the venue for a given `instrument_id`
+                flags = RecordFlag.F_LAST
+
+            delta = OrderBookDelta(
+                instrument_id=instrument_id,
+                action=BookAction.ADD,
+                order=BookOrder(
+                    side=OrderSide.SELL,
+                    price=Price(Decimal(ask.price), price_precision),
+                    size=Quantity(Decimal(ask.size), size_precision),
+                    order_id=0,
+                ),
+                flags=flags,
+                sequence=0,
+                ts_event=ts_event,
+                ts_init=ts_init,
+            )
+            deltas.append(delta)
+
+        return OrderBookDeltas(instrument_id=instrument_id, deltas=deltas)
+
 
 class DYDXWsOrderbookSnapshotChannelData(msgspec.Struct, forbid_unknown_fields=True):
     """
@@ -363,78 +446,13 @@ class DYDXWsOrderbookSnapshotChannelData(msgspec.Struct, forbid_unknown_fields=T
         """
         Parse the order book message into OrderBookDeltas.
         """
-        deltas: list[OrderBookDelta] = []
-
-        # Add initial clear
-        clear = OrderBookDelta.clear(
-            instrument_id=instrument_id,
-            sequence=0,
-            ts_event=ts_event,
-            ts_init=ts_init,
+        return self.contents.parse_to_snapshot(
+            instrument_id,
+            price_precision,
+            size_precision,
+            ts_event,
+            ts_init,
         )
-        deltas.append(clear)
-
-        if self.contents.bids is None:
-            self.contents.bids = []
-
-        if self.contents.asks is None:
-            self.contents.asks = []
-
-        num_bids_raw = len(self.contents.bids)
-        num_asks_raw = len(self.contents.asks)
-
-        for bid_id, bid in enumerate(self.contents.bids):
-            flags = 0
-
-            if bid_id == num_bids_raw - 1 and num_asks_raw == 0:
-                # F_LAST, 1 << 7
-                # Last message in the packet from the venue for a given `instrument_id`
-                flags = RecordFlag.F_LAST
-
-            order = BookOrder(
-                side=OrderSide.BUY,
-                price=Price(Decimal(bid.price), price_precision),
-                size=Quantity(Decimal(bid.size), size_precision),
-                order_id=0,
-            )
-
-            delta = OrderBookDelta(
-                instrument_id=instrument_id,
-                action=BookAction.ADD,
-                order=order,
-                flags=flags,
-                sequence=0,
-                ts_event=ts_event,
-                ts_init=ts_init,
-            )
-
-            deltas.append(delta)
-
-        for ask_id, ask in enumerate(self.contents.asks):
-            flags = 0
-
-            if ask_id == num_asks_raw - 1:
-                # F_LAST, 1 << 7
-                # Last message in the packet from the venue for a given `instrument_id`
-                flags = RecordFlag.F_LAST
-
-            delta = OrderBookDelta(
-                instrument_id=instrument_id,
-                action=BookAction.ADD,
-                order=BookOrder(
-                    side=OrderSide.SELL,
-                    price=Price(Decimal(ask.price), price_precision),
-                    size=Quantity(Decimal(ask.size), size_precision),
-                    order_id=0,
-                ),
-                flags=flags,
-                sequence=0,
-                ts_event=ts_event,
-                ts_init=ts_init,
-            )
-            deltas.append(delta)
-
-        return OrderBookDeltas(instrument_id=instrument_id, deltas=deltas)
 
 
 class DYDXWsOrderbookBatchedData(msgspec.Struct, forbid_unknown_fields=True):
@@ -472,19 +490,18 @@ class DYDXWsOrderbookBatchedData(msgspec.Struct, forbid_unknown_fields=True):
             if deltas_message.asks is None:
                 deltas_message.asks = []
 
-            num_bids = len(deltas_message.bids)
-            num_asks = len(deltas_message.asks)
+            bids_len = len(deltas_message.bids)
+            asks_len = len(deltas_message.asks)
 
-            for bid_id, bid in enumerate(deltas_message.bids):
+            for idx, bid in enumerate(deltas_message.bids):
                 flags = 0
-
                 if (
                     delta_message_id == num_delta_messages - 1
-                    and bid_id == num_bids - 1
-                    and num_asks == 0
+                    and idx == bids_len - 1
+                    and asks_len == 0
                 ):
                     # F_LAST, 1 << 7
-                    # Last message in the packet from the venue for a given `instrument_id`
+                    # Last message in the book event or packet from the venue for a given `instrument_id`
                     flags = RecordFlag.F_LAST
 
                 size = Quantity(Decimal(bid[1]), size_precision)
@@ -505,12 +522,11 @@ class DYDXWsOrderbookBatchedData(msgspec.Struct, forbid_unknown_fields=True):
                 )
                 deltas.append(delta)
 
-            for ask_id, ask in enumerate(deltas_message.asks):
+            for idx, ask in enumerate(deltas_message.asks):
                 flags = 0
-
-                if delta_message_id == num_delta_messages - 1 and ask_id == num_asks - 1:
+                if delta_message_id == num_delta_messages - 1 and idx == asks_len - 1:
                     # F_LAST, 1 << 7
-                    # Last message in the packet from the venue for a given `instrument_id`
+                    # Last message in the book event or packet from the venue for a given `instrument_id`
                     flags = RecordFlag.F_LAST
 
                 size = Quantity(Decimal(ask[1]), size_precision)
@@ -547,14 +563,21 @@ class DYDXWsSubaccountsSubscribedContents(msgspec.Struct, forbid_unknown_fields=
         """
         Create an account balance report.
         """
-        account_balances = []
+        account_balances: list[AccountBalance] = []
 
         if self.subaccount is not None:
-            for asset_position in self.subaccount.assetPositions.values():
-                # Only valid for a margin account
-                locked = Decimal(0)
-                account_balance = asset_position.parse_to_account_balance(locked=locked)
-                account_balances.append(account_balance)
+            currency = Currency.from_str(DEFAULT_CURRENCY)
+            free = Decimal(self.subaccount.freeCollateral)
+            total = Decimal(self.subaccount.equity)
+            locked = total - free
+
+            return [
+                AccountBalance(
+                    total=Money(total, currency),
+                    locked=Money(locked, currency),
+                    free=Money(free, currency),
+                ),
+            ]
 
         return account_balances
 
@@ -841,9 +864,7 @@ class DYDXTradingPerpetualMarketMessage(msgspec.Struct, forbid_unknown_fields=Tr
     clobPairId: str | None = None
     ticker: str | None = None
     marketId: int | None = None
-    status: DYDXPerpetualMarketStatus | None = (
-        None  # 'ACTIVE', 'PAUSED', 'CANCEL_ONLY', 'POST_ONLY', or 'INITIALIZING'
-    )
+    status: DYDXPerpetualMarketStatus | None = None
     baseAsset: str | None = None
     quoteAsset: str | None = None
     initialMarginFraction: str | None = None
@@ -861,6 +882,12 @@ class DYDXTradingPerpetualMarketMessage(msgspec.Struct, forbid_unknown_fields=Tr
     trades24H: int | None = None
     nextFundingRate: str | None = None
     baseOpenInterest: str | None = None
+    marketType: str | None = None
+    openInterestLowerCap: str | None = None
+    openInterestUpperCap: str | None = None
+    tickSize: str | None = None
+    stepSize: str | None = None
+    defaultFundingRate1H: str | None = None
 
 
 class DYDXMarketMessageContents(msgspec.Struct, forbid_unknown_fields=True):
