@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -41,7 +41,7 @@ from nautilus_trader.model.instruments import Equity
 from nautilus_trader.model.instruments import FuturesContract
 from nautilus_trader.model.instruments import IndexInstrument
 from nautilus_trader.model.instruments import Instrument
-from nautilus_trader.model.instruments import OptionsContract
+from nautilus_trader.model.instruments import OptionContract
 from nautilus_trader.model.objects import Currency
 from nautilus_trader.model.objects import Price
 from nautilus_trader.model.objects import Quantity
@@ -50,9 +50,46 @@ from nautilus_trader.model.objects import Quantity
 # fmt: on
 
 VENUE_MEMBERS: Final[dict[str, list[str]]] = {
+    # CME Group Exchanges
     "GLBX": ["CME", "CBOT", "NYMEX", "NYBOT"],
+    # ICE Europe Exchanges
     "IFEU": ["ICEEU", "ICEEUSOFT", "IPE"],
+    # ICE Endex
     "NDEX": ["ENDEX"],
+    # Chicago Mercantile Exchange Segments
+    "XCME": ["CME"],
+    "XCEC": ["CME"],
+    "XFXS": ["CME"],
+    # Chicago Board of Trade Segments
+    "XCBT": ["CBOT"],
+    "CBCM": ["CBOT"],
+    # New York Mercantile Exchange Segments
+    "XNYM": ["NYMEX"],
+    "NYUM": ["NYMEX"],
+    # ICE Futures US (formerly NYBOT)
+    "IFUS": ["NYBOT"],
+    # US Major Exchanges
+    "XNAS": ["NASDAQ"],
+    "XNYS": ["NYSE"],
+    "ARCX": ["ARCA"],
+    "BATS": ["BATS"],
+    "IEXG": ["IEX"],
+    # European Exchanges
+    "XLON": ["LSE"],  # London Stock Exchange
+    "XPAR": ["SBF"],  # Euronext Paris
+    "XETR": ["IBIS"],  # Deutsche Börse
+    # Canadian Exchanges
+    "XTSE": ["TSE"],  # Toronto Stock Exchange
+    "XTSX": ["VENTURE"],  # TSX Venture Exchange
+    # Asia-Pacific Exchanges
+    "XASX": ["ASX"],  # Australian Securities Exchange
+    "XHKF": ["HKFE"],  # Hong Kong Futures Exchange
+    "XSES": ["SGX"],  # Singapore Exchange
+    "XOSE": ["OSE.JPN"],  # Osaka Securities Exchange
+    # Other Derivatives Exchanges
+    "XEUR": ["SOFFEX"],  # Eurex
+    "XSFE": ["SNFE"],  # Sydney Futures Exchange
+    "XMEX": ["MEXDER"],  # Mexican Derivatives Exchange
 }
 
 FUTURES_MONTH_TO_CODE: dict[str, str] = {
@@ -91,27 +128,32 @@ VENUES_CFD = [
 ]
 VENUES_CMDTY = ["IBCMDTY"]  # self named, in fact mapping to "SMART" when parsing
 
-RE_CASH = re.compile(r"^(?P<symbol>[A-Z]{3})\/(?P<currency>[A-Z]{3})$")
-RE_CFD_CASH = re.compile(r"^(?P<symbol>[A-Z]{3})\.(?P<currency>[A-Z]{3})$")
+RE_CASH = re.compile(r"^(?P<symbol>[A-Z]{3})\/(?P<currency>[A-Z]{3})$")  # "EUR/USD"
+RE_CFD_CASH = re.compile(r"^(?P<symbol>[A-Z]{3})\.(?P<currency>[A-Z]{3})$")  # "EUR.USD"
 RE_OPT = re.compile(
     r"^(?P<symbol>^[A-Z. ]{1,6})(?P<expiry>\d{6})(?P<right>[CP])(?P<strike>\d{5})(?P<decimal>\d{3})$",
-)
-RE_FUT_UNDERLYING = re.compile(r"^(?P<symbol>\w{1,3})$")
-RE_FUT = re.compile(r"^(?P<symbol>\w{1,3})(?P<month>[FGHJKMNQUVXZ])(?P<year>\d{2})$")
-RE_FUT_ORIGINAL = re.compile(r"^(?P<symbol>\w{1,3})(?P<month>[FGHJKMNQUVXZ])(?P<year>\d)$")
+)  # "AAPL220617C00155000"
+RE_FUT_UNDERLYING = re.compile(r"^(?P<symbol>\w{1,3})$")  # "ES"
+RE_FUT = re.compile(r"^(?P<symbol>\w{1,3})(?P<month>[FGHJKMNQUVXZ])(?P<year>\d{2})$")  # "ESM23"
+RE_FUT_ORIGINAL = re.compile(
+    r"^(?P<symbol>\w{1,3})(?P<month>[FGHJKMNQUVXZ])(?P<year>\d)$",
+)  # "ESM3"
 RE_FUT2 = re.compile(
     r"^(?P<symbol>\w{1,4})(?P<month>(JAN|FEB|MAR|APR|MAY|JUN|JUL|AUG|SEP|OCT|NOV|DEC))(?P<year>\d{2})$",
-)
+)  # "ESMAR23"
 RE_FUT2_ORIGINAL = re.compile(
     r"^(?P<symbol>\w{1,4}) *(?P<month>(JAN|FEB|MAR|APR|MAY|JUN|JUL|AUG|SEP|OCT|NOV|DEC)) (?P<year>\d{2})$",
-)
+)  # "ES MAR 23"
+RE_FUT3_ORIGINAL = re.compile(
+    r"^(?P<symbol>[A-Z]+)(?P<year>\d{2})(?P<month>(JAN|FEB|MAR|APR|MAY|JUN|JUL|AUG|SEP|OCT|NOV|DEC))FUT$",
+)  # "NIFTY25MARFUT"
 RE_FOP = re.compile(
     r"^(?P<symbol>\w{1,3})(?P<month>[FGHJKMNQUVXZ])(?P<year>\d{2})(?P<right>[CP])(?P<strike>.{4,5})$",
-)
+)  # "ESM23C4200"
 RE_FOP_ORIGINAL = re.compile(
     r"^(?P<symbol>\w{1,3})(?P<month>[FGHJKMNQUVXZ])(?P<year>\d)\s(?P<right>[CP])(?P<strike>\d{1,4}(?:\.\d)?)$",
-)
-RE_CRYPTO = re.compile(r"^(?P<symbol>[A-Z]*)\/(?P<currency>[A-Z]{3})$")
+)  # "ESM3 C4420"
+RE_CRYPTO = re.compile(r"^(?P<symbol>[A-Z]*)\/(?P<currency>[A-Z]{3})$")  # "BTC/USD"
 
 
 def _extract_isin(details: IBContractDetails) -> int:
@@ -163,7 +205,7 @@ def parse_instrument(
     elif security_type in ("FUT", "CONTFUT"):
         return parse_futures_contract(details=contract_details, instrument_id=instrument_id)
     elif security_type in ("OPT", "FOP"):
-        return parse_options_contract(details=contract_details, instrument_id=instrument_id)
+        return parse_option_contract(details=contract_details, instrument_id=instrument_id)
     elif security_type == "CASH":
         return parse_forex_contract(details=contract_details, instrument_id=instrument_id)
     elif security_type == "CRYPTO":
@@ -269,10 +311,10 @@ def parse_futures_contract(
     )
 
 
-def parse_options_contract(
+def parse_option_contract(
     details: IBContractDetails,
     instrument_id: InstrumentId,
-) -> OptionsContract:
+) -> OptionContract:
     price_precision: int = _tick_size_to_precision(details.minTick)
     timestamp = time.time_ns()
     asset_class = sec_type_to_asset_class(details.underSecType)
@@ -283,7 +325,7 @@ def parse_options_contract(
     expiration = expiry_timestring_to_datetime(details.contract.lastTradeDateOrContractMonth)
     activation = expiration - pd.Timedelta(days=90)  # TODO: Make this more accurate
 
-    return OptionsContract(
+    return OptionContract(
         instrument_id=instrument_id,
         raw_symbol=Symbol(details.contract.localSymbol),
         asset_class=asset_class,
@@ -524,6 +566,9 @@ def ib_contract_to_instrument_id_simplified_symbology(  # noqa: C901 (too comple
     elif security_type == "FUT" and (m := RE_FUT2_ORIGINAL.match(contract.localSymbol)):
         symbol = f"{m['symbol']}{FUTURES_MONTH_TO_CODE[m['month']]}{m['year']}"
         venue = contract.exchange
+    elif security_type == "FUT" and (m := RE_FUT3_ORIGINAL.match(contract.localSymbol)):
+        symbol = f"{m['symbol']}{FUTURES_MONTH_TO_CODE[m['month']]}{m['year']}"
+        venue = contract.exchange
     elif security_type == "FOP" and (m := RE_FOP_ORIGINAL.match(contract.localSymbol)):
         symbol = f"{m['symbol']}{m['month']}{decade_digit(m['year'], contract)}{m['year']}{m['right']}{m['strike']}"
         venue = contract.exchange
