@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -24,6 +24,7 @@ from nautilus_trader.adapters.polymarket.common.enums import PolymarketOrderStat
 from nautilus_trader.adapters.polymarket.common.enums import PolymarketOrderType
 from nautilus_trader.adapters.polymarket.common.symbol import get_polymarket_instrument_id
 from nautilus_trader.adapters.polymarket.common.symbol import get_polymarket_token_id
+from nautilus_trader.adapters.polymarket.schemas.book import PolymarketTickSizeChange
 from nautilus_trader.model.currencies import USDC
 from nautilus_trader.model.enums import AssetClass
 from nautilus_trader.model.enums import LiquiditySide
@@ -93,7 +94,7 @@ def parse_instrument(
     raw_symbol = Symbol(get_polymarket_token_id(instrument_id))
     description = market_info["question"]
     price_increment = Price.from_str(str(market_info["minimum_tick_size"]))
-    min_quantity = Quantity.from_int(market_info["minimum_order_size"])
+    min_quantity = Quantity.from_int(int(market_info["minimum_order_size"]))
     # size_increment can be 0.01 or 0.001 (precision 2 or 3). Need to determine a reliable solution
     # trades are reported with USDC.e increments though - so we use that here
     size_increment = Quantity.from_str("0.000001")
@@ -128,3 +129,33 @@ def parse_instrument(
         ts_init=ts_init,
         info=market_info,
     )
+
+
+def update_instrument(
+    instrument: BinaryOption,
+    change: PolymarketTickSizeChange,
+    ts_init: int,
+) -> BinaryOption:
+    price_increment = Price.from_str(change.new_tick_size)
+
+    return BinaryOption(
+        instrument_id=instrument.instrument_id,
+        raw_symbol=instrument.raw_symbol,
+        outcome=instrument.outcome,
+        description=instrument.description,
+        asset_class=AssetClass.ALTERNATIVE,
+        currency=USDC,
+        price_increment=price_increment,
+        price_precision=price_increment.precision,
+        size_increment=instrument.size_increment,
+        size_precision=instrument.size_precision,
+        activation_ns=0,  # TBD?
+        expiration_ns=instrument.expiration_ns,
+        max_quantity=None,
+        min_quantity=instrument.min_quantity,
+        maker_fee=instrument.maker_fee,
+        taker_fee=instrument.taker_fee,
+        ts_event=ts_init,
+        ts_init=ts_init,
+        info=instrument.info,
+    )
