@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -18,17 +18,18 @@ from collections.abc import Coroutine
 from typing import Any
 
 import msgspec
-import pandas as pd
 from py_clob_client.client import ClobClient
 
 from nautilus_trader.adapters.polymarket.common.constants import POLYMARKET_VENUE
 from nautilus_trader.adapters.polymarket.common.deltas import compute_effective_deltas
+from nautilus_trader.adapters.polymarket.common.parsing import update_instrument
 from nautilus_trader.adapters.polymarket.common.symbol import get_polymarket_instrument_id
 from nautilus_trader.adapters.polymarket.common.symbol import get_polymarket_token_id
 from nautilus_trader.adapters.polymarket.config import PolymarketDataClientConfig
 from nautilus_trader.adapters.polymarket.providers import PolymarketInstrumentProvider
 from nautilus_trader.adapters.polymarket.schemas.book import PolymarketBookSnapshot
 from nautilus_trader.adapters.polymarket.schemas.book import PolymarketQuotes
+from nautilus_trader.adapters.polymarket.schemas.book import PolymarketTickSizeChange
 from nautilus_trader.adapters.polymarket.schemas.book import PolymarketTrade
 from nautilus_trader.adapters.polymarket.websocket.client import PolymarketWebSocketChannel
 from nautilus_trader.adapters.polymarket.websocket.client import PolymarketWebSocketClient
@@ -37,16 +38,26 @@ from nautilus_trader.cache.cache import Cache
 from nautilus_trader.common.component import LiveClock
 from nautilus_trader.common.component import MessageBus
 from nautilus_trader.common.enums import LogColor
-from nautilus_trader.core.uuid import UUID4
+from nautilus_trader.data.messages import RequestBars
+from nautilus_trader.data.messages import RequestInstrument
+from nautilus_trader.data.messages import RequestInstruments
+from nautilus_trader.data.messages import RequestQuoteTicks
+from nautilus_trader.data.messages import RequestTradeTicks
+from nautilus_trader.data.messages import SubscribeBars
+from nautilus_trader.data.messages import SubscribeOrderBook
+from nautilus_trader.data.messages import SubscribeQuoteTicks
+from nautilus_trader.data.messages import SubscribeTradeTicks
+from nautilus_trader.data.messages import UnsubscribeBars
+from nautilus_trader.data.messages import UnsubscribeOrderBook
+from nautilus_trader.data.messages import UnsubscribeQuoteTicks
+from nautilus_trader.data.messages import UnsubscribeTradeTicks
 from nautilus_trader.live.data_client import LiveMarketDataClient
 from nautilus_trader.model.book import OrderBook
-from nautilus_trader.model.data import BarType
 from nautilus_trader.model.data import OrderBookDeltas
 from nautilus_trader.model.data import QuoteTick
 from nautilus_trader.model.enums import BookType
 from nautilus_trader.model.identifiers import ClientId
 from nautilus_trader.model.identifiers import InstrumentId
-from nautilus_trader.model.identifiers import Venue
 from nautilus_trader.model.instruments import BinaryOption
 
 
@@ -98,6 +109,9 @@ class PolymarketDataClient(LiveMarketDataClient):
 
         # Configuration
         self._config = config
+        self._log.info(f"{config.signature_type=}", LogColor.BLUE)
+        self._log.info(f"{config.funder=}", LogColor.BLUE)
+        self._log.info(f"{config.ws_connection_initial_delay_secs=}", LogColor.BLUE)
         self._log.info(f"{config.ws_connection_delay_secs=}", LogColor.BLUE)
         self._log.info(f"{config.update_instruments_interval_mins=}", LogColor.BLUE)
         self._log.info(f"{config.compute_effective_deltas=}", LogColor.BLUE)
@@ -106,16 +120,14 @@ class PolymarketDataClient(LiveMarketDataClient):
         self._http_client = http_client
 
         # WebSocket API
-        self._ws_base_url = self._config.base_url_ws
-        self._ws_client: PolymarketWebSocketClient = self._create_websocket_client()
-        self._ws_clients: dict[InstrumentId, PolymarketWebSocketClient] = {}
+        self._ws_clients: list[PolymarketWebSocketClient] = []
+        self._ws_client_pending_connection: PolymarketWebSocketClient | None = None
+
         self._decoder_market_msg = msgspec.json.Decoder(MARKET_WS_MESSAGE)
 
         # Tasks
-        self._update_instruments_interval_mins: int | None = config.update_instruments_interval_mins
         self._update_instruments_task: asyncio.Task | None = None
-        self._main_ws_connect_task: asyncio.Task | None = None
-        self._main_ws_delay = True
+        self._delayed_ws_client_connection_task: asyncio.Task | None = None
 
         # Hot caches
         self._last_quotes: dict[InstrumentId, QuoteTick] = {}
@@ -126,62 +138,43 @@ class PolymarketDataClient(LiveMarketDataClient):
         await self._instrument_provider.initialize()
         self._send_all_instruments_to_data_engine()
 
-        if self._update_instruments_interval_mins:
+        if self._config.update_instruments_interval_mins:
             self._update_instruments_task = self.create_task(
-                self._update_instruments(self._update_instruments_interval_mins),
+                self._update_instruments(self._config.update_instruments_interval_mins),
             )
 
-        self._main_ws_connect_task = self.create_task(self._connect_main_ws_after_delay())
-
     async def _disconnect(self) -> None:
         if self._update_instruments_task:
             self._log.debug("Canceling task 'update_instruments'")
             self._update_instruments_task.cancel()
             self._update_instruments_task = None
 
-        if self._main_ws_connect_task:
-            self._log.debug("Canceling task 'connect_main_ws_after_delay'")
-            self._main_ws_connect_task.cancel()
-            self._main_ws_connect_task = None
+        if self._delayed_ws_client_connection_task:
+            self._log.debug("Canceling task 'delayed_ws_client_connection'")
+            self._delayed_ws_client_connection_task.cancel()
+            self._delayed_ws_client_connection_task = None
 
         # Shutdown websockets
         tasks: set[Coroutine[Any, Any, None]] = set()
 
-        if self._ws_client.is_connected():
-            tasks.add(self._ws_client.disconnect())
-
-        for ws_client in self._ws_clients.values():
+        for ws_client in self._ws_clients:
             if ws_client.is_connected():
                 tasks.add(ws_client.disconnect())
 
         if tasks:
             await asyncio.gather(*tasks)
 
-        self._main_ws_delay = True
-
     def _create_websocket_client(self) -> PolymarketWebSocketClient:
         self._log.info("Creating new PolymarketWebSocketClient", LogColor.MAGENTA)
         return PolymarketWebSocketClient(
             self._clock,
-            base_url=self._ws_base_url,
+            base_url=self._config.base_url_ws,
             channel=PolymarketWebSocketChannel.MARKET,
             handler=self._handle_ws_message,
             handler_reconnect=None,
             loop=self._loop,
         )
 
-    async def _connect_main_ws_after_delay(self) -> None:
-        delay_secs = self._config.ws_connection_delay_secs
-        self._log.info(
-            f"Awaiting initial websocket connection delay ({delay_secs}s)...",
-            LogColor.BLUE,
-        )
-        await asyncio.sleep(delay_secs)
-        if self._ws_client.asset_subscriptions():
-            await self._ws_client.connect()
-
-        self._main_ws_delay = False
-
     def _send_all_instruments_to_data_engine(self) -> None:
         for instrument in self._instrument_provider.get_all().values():
             self._handle_data(instrument)
@@ -201,32 +194,44 @@ class PolymarketDataClient(LiveMarketDataClient):
         except asyncio.CancelledError:
             self._log.debug("Canceled task 'update_instruments'")
 
-    async def _subscribe_asset_book(self, instrument_id):
-        token_id = get_polymarket_token_id(instrument_id)
-
-        if not self._ws_client.is_connected():
-            ws_client = self._ws_client
-            if token_id in ws_client.asset_subscriptions():
-                return  # Already subscribed
-            ws_client.subscribe_book(asset=token_id)
-            if not self._main_ws_delay:
-                await ws_client.connect()
-        else:
-            ws_client = self._create_websocket_client()
-            if token_id in ws_client.asset_subscriptions():
-                return  # Already subscribed
-            self._ws_clients[instrument_id] = ws_client
-            ws_client.subscribe_book(asset=token_id)
-            await ws_client.connect()
-
-    async def _subscribe_order_book_deltas(
+    async def _delayed_ws_client_connection(
         self,
-        instrument_id: InstrumentId,
-        book_type: BookType,
-        depth: int | None = None,
-        params: dict[str, Any] | None = None,
+        ws_client: PolymarketWebSocketClient,
+        sleep_secs: int,
     ) -> None:
-        if book_type == BookType.L3_MBO:
+        try:
+            await asyncio.sleep(sleep_secs)
+            self._ws_clients.append(ws_client)
+            await ws_client.connect()
+        finally:
+            self._ws_client_pending_connection = None
+            self._delayed_ws_client_connection_task = None
+
+    async def _subscribe_asset_book(self, instrument_id):
+        create_connect_task = False
+        if self._ws_client_pending_connection is None:
+            self._ws_client_pending_connection = self._create_websocket_client()
+            create_connect_task = True
+
+        token_id = get_polymarket_token_id(instrument_id)
+        self._ws_client_pending_connection.subscribe_book(token_id)
+
+        if create_connect_task:
+            self._delayed_ws_client_connection_task = self.create_task(
+                self._delayed_ws_client_connection(
+                    self._ws_client_pending_connection,
+                    (
+                        self._config.ws_connection_delay_secs
+                        if self._ws_clients
+                        else self._config.ws_connection_initial_delay_secs
+                    ),
+                ),
+                log_msg="Delayed start PolymarketWebSocketClient connection",
+                success_msg="Finished delaying start of PolymarketWebSocketClient connection",
+            )
+
+    async def _subscribe_order_book_deltas(self, command: SubscribeOrderBook) -> None:
+        if command.book_type == BookType.L3_MBO:
             self._log.error(
                 "Cannot subscribe to order book deltas: "
                 "L3_MBO data is not published by Polymarket. "
@@ -235,164 +240,99 @@ class PolymarketDataClient(LiveMarketDataClient):
             return
 
         if self._config.compute_effective_deltas:
-            local_book = OrderBook(instrument_id, book_type=BookType.L2_MBP)
-            self._local_books[instrument_id] = local_book
+            local_book = OrderBook(command.instrument_id, book_type=BookType.L2_MBP)
+            self._local_books[command.instrument_id] = local_book
 
-        await self._subscribe_asset_book(instrument_id)
+        await self._subscribe_asset_book(command.instrument_id)
 
-    async def _subscribe_quote_ticks(
-        self,
-        instrument_id: InstrumentId,
-        params: dict[str, Any] | None = None,
-    ) -> None:
-        await self._subscribe_asset_book(instrument_id)
+    async def _subscribe_quote_ticks(self, command: SubscribeQuoteTicks) -> None:
+        await self._subscribe_asset_book(command.instrument_id)
 
-    async def _subscribe_trade_ticks(
-        self,
-        instrument_id: InstrumentId,
-        params: dict[str, Any] | None = None,
-    ) -> None:
-        await self._subscribe_asset_book(instrument_id)
+    async def _subscribe_trade_ticks(self, command: SubscribeTradeTicks) -> None:
+        await self._subscribe_asset_book(command.instrument_id)
 
-    async def _subscribe_bars(
-        self,
-        bar_type: BarType,
-        params: dict[str, Any] | None = None,
-    ) -> None:
+    async def _subscribe_bars(self, command: SubscribeBars) -> None:
         self._log.error(
-            f"Cannot subscribe to {bar_type} bars: not implemented for Polymarket",
+            f"Cannot subscribe to {command.bar_type} bars: not implemented for Polymarket",
         )
 
-    async def _unsubscribe_order_book_deltas(
-        self,
-        instrument_id: InstrumentId,
-        params: dict[str, Any] | None = None,
-    ) -> None:
+    async def _unsubscribe_order_book_deltas(self, command: UnsubscribeOrderBook) -> None:
         self._log.error(
-            f"Cannot unsubscribe from {instrument_id} order book deltas: unsubscribing not supported by Polymarket",
+            f"Cannot unsubscribe from {command.instrument_id} order book deltas: unsubscribing not supported by Polymarket",
         )
 
-    async def _unsubscribe_order_book_snapshots(
-        self,
-        instrument_id: InstrumentId,
-        params: dict[str, Any] | None = None,
-    ) -> None:
+    async def _unsubscribe_order_book_snapshots(self, command: UnsubscribeOrderBook) -> None:
         self._log.error(
-            f"Cannot unsubscribe from {instrument_id} order book snapshots: unsubscribing not supported by Polymarket",
+            f"Cannot unsubscribe from {command.instrument_id} order book snapshots: unsubscribing not supported by Polymarket",
         )
 
-    async def _unsubscribe_quote_ticks(
-        self,
-        instrument_id: InstrumentId,
-        params: dict[str, Any] | None = None,
-    ) -> None:
+    async def _unsubscribe_quote_ticks(self, command: UnsubscribeQuoteTicks) -> None:
         self._log.error(
-            f"Cannot unsubscribe from {instrument_id} quotes: unsubscribing not supported by Polymarket",
+            f"Cannot unsubscribe from {command.instrument_id} quotes: unsubscribing not supported by Polymarket",
         )
 
-    async def _unsubscribe_trade_ticks(
-        self,
-        instrument_id: InstrumentId,
-        params: dict[str, Any] | None = None,
-    ) -> None:
+    async def _unsubscribe_trade_ticks(self, command: UnsubscribeTradeTicks) -> None:
         self._log.error(
-            f"Cannot unsubscribe from {instrument_id} trades: unsubscribing not supported by Polymarket",
+            f"Cannot unsubscribe from {command.instrument_id} trades: unsubscribing not supported by Polymarket",
         )
 
-    async def _unsubscribe_bars(
-        self,
-        bar_type: BarType,
-        params: dict[str, Any] | None = None,
-    ) -> None:
+    async def _unsubscribe_bars(self, command: UnsubscribeBars) -> None:
         self._log.error(
-            f"Cannot unsubscribe from {bar_type} bars: not implemented for Polymarket",
+            f"Cannot unsubscribe from {command.bar_type} bars: not implemented for Polymarket",
         )
 
-    async def _request_instrument(
-        self,
-        instrument_id: InstrumentId,
-        correlation_id: UUID4,
-        start: pd.Timestamp | None = None,
-        end: pd.Timestamp | None = None,
-        params: dict[str, Any] | None = None,
-    ) -> None:
-        if start is not None:
+    async def _request_instrument(self, request: RequestInstrument) -> None:
+        if request.start is not None:
             self._log.warning(
-                f"Requesting instrument {instrument_id} with specified `start` which has no effect",
+                f"Requesting instrument {request.instrument_id} with specified `start` which has no effect",
             )
 
-        if end is not None:
+        if request.end is not None:
             self._log.warning(
-                f"Requesting instrument {instrument_id} with specified `end` which has no effect",
+                f"Requesting instrument {request.instrument_id} with specified `end` which has no effect",
             )
 
-        instrument: BinaryOption | None = self._instrument_provider.find(instrument_id)
+        instrument: BinaryOption | None = self._instrument_provider.find(request.instrument_id)
         if instrument is None:
-            self._log.error(f"Cannot find instrument for {instrument_id}")
+            self._log.error(f"Cannot find instrument for {request.instrument_id}")
             return
 
-        self._handle_instrument(instrument, correlation_id, params)
+        self._handle_instrument(instrument, request.id, request.params)
 
-    async def _request_instruments(
-        self,
-        venue: Venue,
-        correlation_id: UUID4,
-        start: pd.Timestamp | None = None,
-        end: pd.Timestamp | None = None,
-        params: dict[str, Any] | None = None,
-    ) -> None:
-        if start is not None:
+    async def _request_instruments(self, request: RequestInstruments) -> None:
+        if request.start is not None:
             self._log.warning(
-                f"Requesting instruments for {venue} with specified `start` which has no effect",
+                f"Requesting instruments for {request.venue} with specified `start` which has no effect",
             )
 
-        if end is not None:
+        if request.end is not None:
             self._log.warning(
-                f"Requesting instruments for {venue} with specified `end` which has no effect",
+                f"Requesting instruments for {request.venue} with specified `end` which has no effect",
             )
 
         all_instruments = self._instrument_provider.get_all()
         target_instruments = []
         for instrument in all_instruments.values():
-            if instrument.venue == venue:
+            if instrument.venue == request.venue:
                 target_instruments.append(instrument)
 
-        self._handle_instruments(target_instruments, venue, correlation_id, params)
+        self._handle_instruments(
+            target_instruments,
+            request.venue,
+            request.id,
+            request.params,
+        )
 
-    async def _request_quote_ticks(
-        self,
-        instrument_id: InstrumentId,
-        limit: int,
-        correlation_id: UUID4,
-        start: pd.Timestamp | None = None,
-        end: pd.Timestamp | None = None,
-        params: dict[str, Any] | None = None,
-    ) -> None:
+    async def _request_quote_ticks(self, request: RequestQuoteTicks) -> None:
         self._log.error("Cannot request historical quotes: not published by Polymarket")
 
-    async def _request_trade_ticks(
-        self,
-        instrument_id: InstrumentId,
-        limit: int,
-        correlation_id: UUID4,
-        start: pd.Timestamp | None = None,
-        end: pd.Timestamp | None = None,
-        params: dict[str, Any] | None = None,
-    ) -> None:
+    async def _request_trade_ticks(self, request: RequestTradeTicks) -> None:
         self._log.error("Cannot request historical trades: not published by Polymarket")
 
-    async def _request_bars(
-        self,
-        bar_type: BarType,
-        limit: int,
-        correlation_id: UUID4,
-        start: pd.Timestamp | None = None,
-        end: pd.Timestamp | None = None,
-        params: dict[str, Any] | None = None,
-    ) -> None:
+    async def _request_bars(self, request: RequestBars) -> None:
         self._log.error("Cannot request historical bars: not published by Polymarket")
 
-    def _handle_ws_message(self, raw: bytes) -> None:
+    def _handle_ws_message(self, raw: bytes) -> None:  # noqa: C901 (too complex)
         # Uncomment for development
         # self._log.info(str(raw), LogColor.MAGENTA)
         try:
@@ -419,6 +359,8 @@ class PolymarketDataClient(LiveMarketDataClient):
                         self._handle_quote(instrument=instrument, ws_message=msg)
                     elif isinstance(msg, PolymarketTrade):
                         self._handle_trade(instrument=instrument, ws_message=msg)
+                    elif isinstance(msg, PolymarketTickSizeChange):
+                        self._handle_instrument_update(instrument=instrument, ws_message=msg)
                     else:
                         self._log.error(f"Unknown websocket message topic: {ws_message}")
         except Exception as e:
@@ -504,3 +446,13 @@ class PolymarketDataClient(LiveMarketDataClient):
         now_ns = self._clock.timestamp_ns()
         trade = ws_message.parse_to_trade_tick(instrument=instrument, ts_init=now_ns)
         self._handle_data(trade)
+
+    def _handle_instrument_update(
+        self,
+        instrument: BinaryOption,
+        ws_message: PolymarketTickSizeChange,
+    ) -> None:
+        now_ns = self._clock.timestamp_ns()
+        instrument = update_instrument(instrument, change=ws_message, ts_init=now_ns)
+        self._log.warning(f"Instrument tick size changed: {instrument}")
+        self._handle_data(instrument)
