@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -29,7 +29,7 @@ from nautilus_trader.model.instruments import CryptoFuture
 from nautilus_trader.model.instruments import CryptoPerpetual
 from nautilus_trader.model.instruments import CurrencyPair
 from nautilus_trader.model.instruments import Instrument
-from nautilus_trader.model.instruments import OptionsContract
+from nautilus_trader.model.instruments import OptionContract
 
 
 def create_instrument_info(instrument: Instrument) -> nautilus_pyo3.InstrumentMiniInfo:
@@ -48,28 +48,32 @@ def infer_tardis_exchange_str(instrument: Instrument) -> str:  # noqa: C901 (too
         case "BINANCE":
             if isinstance(instrument, CurrencyPair):
                 return "binance"
-            elif isinstance(instrument, OptionsContract):
+            elif isinstance(instrument, OptionContract):
                 return "binance-options"
             else:
                 return "binance-futures"
+        case "BINANCE_US":
+            return "binance-us"
+        case "BINANCE_DELIVERY":
+            return "binance-delivery"
         case "BITFINEX":
             if isinstance(instrument, CurrencyPair):
                 return "bitfinex"
             else:
                 return "bitfinex-derivatives"
+        case "BLOCKCHAIN_COM":
+            return "blockchain-com"
         case "BYBIT":
             if isinstance(instrument, CurrencyPair):
                 return "bybit-spot"
-            elif isinstance(instrument, OptionsContract):
+            elif isinstance(instrument, OptionContract):
                 return "bybit-options"
             else:
                 return "bybit"
         case "CRYPTO_COM":
             if isinstance(instrument, CurrencyPair):
                 return "crypto-com"
-            else:
-                return "crypto-com-derivatives"
-        case "GATEIO":
+        case "GATE_IO":
             if isinstance(instrument, CurrencyPair):
                 return "gate-io"
             else:
@@ -79,7 +83,7 @@ def infer_tardis_exchange_str(instrument: Instrument) -> str:  # noqa: C901 (too
                 return "huobi"
             elif isinstance(instrument, CryptoPerpetual):
                 return "huobi-dm-linear-swap"
-            elif isinstance(instrument, OptionsContract):
+            elif isinstance(instrument, OptionContract):
                 return "huobi-dm-options"
             else:
                 return "huobi-dm-swap"
@@ -95,7 +99,7 @@ def infer_tardis_exchange_str(instrument: Instrument) -> str:  # noqa: C901 (too
                 return "okex-swap"
             elif isinstance(instrument, CryptoFuture):
                 return "okex-futures"
-            elif isinstance(instrument, OptionsContract):
+            elif isinstance(instrument, OptionContract):
                 return "okex-options"
 
     return venue.lower()
