@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -171,7 +171,7 @@ class BacktestNode:
                 # Broad catch all prevents a single backtest run from halting
                 # the execution of the other backtests (such as a zero balance exception).
                 if not is_logging_initialized():
-                    init_logging()
+                    _guard = init_logging()
                 log = Logger(type(self).__name__)
                 log.error(f"Error running backtest: {e}")
                 log.info(f"Config: {config}")
@@ -269,6 +269,9 @@ class BacktestNode:
                 use_position_ids=config.use_position_ids,
                 use_random_ids=config.use_random_ids,
                 use_reduce_only=config.use_reduce_only,
+                bar_execution=config.bar_execution,
+                bar_adaptive_high_low_ordering=config.bar_adaptive_high_low_ordering,
+                trade_execution=config.trade_execution,
             )
 
         # Add instruments
