@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -28,6 +28,7 @@ from msgspec import Meta
 from nautilus_trader.common import Environment
 from nautilus_trader.core.correctness import PyCondition
 from nautilus_trader.core.uuid import UUID4
+from nautilus_trader.model.data import BarSpecification
 from nautilus_trader.model.data import BarType
 from nautilus_trader.model.identifiers import ComponentId
 from nautilus_trader.model.identifiers import Identifier
@@ -78,13 +79,15 @@ def resolve_config_path(path: str) -> type[NautilusConfig]:
     return config
 
 
-def msgspec_encoding_hook(obj: Any) -> Any:
+def msgspec_encoding_hook(obj: Any) -> Any:  # noqa: C901 (too complex)
     if isinstance(obj, Decimal):
         return str(obj)
     if isinstance(obj, UUID4):
         return obj.value
     if isinstance(obj, Identifier):
         return obj.value
+    if isinstance(obj, BarSpecification):
+        return str(obj)
     if isinstance(obj, BarType):
         return str(obj)
     if isinstance(obj, (Price | Quantity)):
@@ -102,13 +105,17 @@ def msgspec_encoding_hook(obj: Any) -> Any:
     raise TypeError(f"Encoding objects of type {obj.__class__} is unsupported")
 
 
-def msgspec_decoding_hook(obj_type: type, obj: Any) -> Any:
-    if obj_type in (Decimal, UUID4, pd.Timestamp, pd.Timedelta):
+def msgspec_decoding_hook(obj_type: type, obj: Any) -> Any:  # noqa: C901 (too complex)
+    if obj_type in (Decimal, pd.Timestamp, pd.Timedelta):
         return obj_type(obj)
+    if obj_type == UUID4:
+        return UUID4.from_str(obj)
     if obj_type == InstrumentId:
         return InstrumentId.from_str(obj)
     if issubclass(obj_type, Identifier):
         return obj_type(obj)
+    if obj_type == BarSpecification:
+        return BarSpecification.from_str(obj)
     if obj_type == BarType:
         return BarType.from_str(obj)
     if obj_type == Price:
@@ -254,7 +261,7 @@ class DatabaseConfig(NautilusConfig, frozen=True):
         The account password for the database connection.
         If a value is provided then it will be redacted in the string repr for this object.
     ssl : bool, default False
-        If database should use an SSL enabled connection.
+        If socket should use an SSL (TLS encryption) enabled connection.
     timeout : int, default 20
         The timeout (seconds) to wait for a new connection.
 
@@ -361,7 +368,7 @@ class InstrumentProviderConfig(NautilusConfig, frozen=True):
     load_all : bool, default False
         If all venue instruments should be loaded on start.
     load_ids : frozenset[InstrumentId], optional
-        The list of instrument IDs to be loaded on start (if `load_all_instruments` is False).
+        The list of instrument IDs to be loaded on start (if `load_all` is False).
     filters : frozendict or dict[str, Any], optional
         The venue specific instrument loading filters to apply.
     filter_callable: str, optional
@@ -413,10 +420,17 @@ class ActorConfig(NautilusConfig, kw_only=True, frozen=True):
     component_id : ComponentId, optional
         The component ID. If ``None`` then the identifier will be taken from
         `type(self).__name__`.
+    log_events : bool, default True
+        If events should be logged by the actor.
+        If False, then only warning events and above are logged.
+    log_commands : bool, default True
+        If commands should be logged by the actor.
 
     """
 
     component_id: ComponentId | None = None
+    log_events: bool = True
+    log_commands: bool = True
 
 
 class ImportableActorConfig(NautilusConfig, frozen=True):
