@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -46,7 +46,7 @@ class InstrumentProvider:
         self._instruments: dict[InstrumentId, Instrument] = {}
         self._currencies: dict[str, Currency] = {}
 
-        # Settings
+        # Configuration
         self._load_all_on_start = config.load_all
         self._load_ids_on_start = set(config.load_ids) if config.load_ids is not None else None
         self._filters = config.filters
@@ -160,12 +160,14 @@ class InstrumentProvider:
             while self._loading:
                 await asyncio.sleep(0.1)
 
-        if not self._loaded:
-            self._log.info("Initializing instruments")
+        if not reload and self._loaded:
+            return  # Already loaded
 
-            # Set state flag
-            self._loading = True
+        # Set state flag
+        self._loading = True
+        self._log.info("Initializing instruments...")
 
+        try:
             if self._load_all_on_start:
                 await self.load_all_async(self._filters)
             elif self._load_ids_on_start:
@@ -179,6 +181,10 @@ class InstrumentProvider:
                 self._log.info(f"Loading instruments: {instruments_str}{filters_str}")
 
                 await self.load_ids_async(instrument_ids, self._filters)
+        except Exception as e:
+            # Catch unexpected exception to ensure that the self._loading flag
+            # is reset to False
+            self._log.error(f"Failed to load the instruments: {e}")
 
         if self._instruments:
             self._log.info(f"Loaded {self.count} instruments")
