@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -14,6 +14,7 @@
 # -------------------------------------------------------------------------------------------------
 
 import asyncio
+import traceback
 from asyncio import Queue
 from typing import Final
 
@@ -25,8 +26,9 @@ from nautilus_trader.core.correctness import PyCondition
 from nautilus_trader.core.data import Data
 from nautilus_trader.data.engine import DataEngine
 from nautilus_trader.data.messages import DataCommand
-from nautilus_trader.data.messages import DataRequest
 from nautilus_trader.data.messages import DataResponse
+from nautilus_trader.data.messages import RequestData
+from nautilus_trader.live.enqueue import ThrottledEnqueuer
 
 
 class LiveDataEngine(DataEngine):
@@ -79,6 +81,35 @@ class LiveDataEngine(DataEngine):
         self._res_queue: asyncio.Queue = Queue(maxsize=config.qsize)
         self._data_queue: asyncio.Queue = Queue(maxsize=config.qsize)
 
+        self._cmd_enqueuer: ThrottledEnqueuer[DataCommand] = ThrottledEnqueuer(
+            qname="cmd_queue",
+            queue=self._cmd_queue,
+            loop=self._loop,
+            clock=self._clock,
+            logger=self._log,
+        )
+        self._req_enqueuer: ThrottledEnqueuer[RequestData] = ThrottledEnqueuer(
+            qname="req_queue",
+            queue=self._req_queue,
+            loop=self._loop,
+            clock=self._clock,
+            logger=self._log,
+        )
+        self._res_enqueuer: ThrottledEnqueuer[DataResponse] = ThrottledEnqueuer(
+            qname="res_queue",
+            queue=self._res_queue,
+            loop=self._loop,
+            clock=self._clock,
+            logger=self._log,
+        )
+        self._data_enqueuer: ThrottledEnqueuer[Data] = ThrottledEnqueuer(
+            qname="data_queue",
+            queue=self._data_queue,
+            loop=self._loop,
+            clock=self._clock,
+            logger=self._log,
+        )
+
         # Async tasks
         self._cmd_queue_task: asyncio.Task | None = None
         self._req_queue_task: asyncio.Task | None = None
@@ -169,7 +200,7 @@ class LiveDataEngine(DataEngine):
 
     def req_qsize(self) -> int:
         """
-        Return the number of `DataRequest` objects buffered on the internal queue.
+        Return the number of `RequestData` objects buffered on the internal queue.
 
         Returns
         -------
@@ -228,101 +259,57 @@ class LiveDataEngine(DataEngine):
         """
         Execute the given data command.
 
-        If the internal queue is already full then will log a warning and block
-        until queue size reduces.
+        If the internal queue is at or near capacity, it logs a warning (throttled)
+        and schedules an asynchronous `put()` operation. This ensures all messages are
+        eventually enqueued and processed without blocking the caller when the queue is full.
 
         Parameters
         ----------
         command : DataCommand
             The command to execute.
 
-        Warnings
-        --------
-        This method is not thread-safe and should only be called from the same thread the event
-        loop is running on. Calling it from a different thread may lead to unexpected behavior.
-
         """
-        PyCondition.not_none(command, "command")
-        # Do not allow None through (None is a sentinel value which stops the queue)
+        self._cmd_enqueuer.enqueue(command)
 
-        try:
-            self._loop.call_soon_threadsafe(self._cmd_queue.put_nowait, command)
-        except asyncio.QueueFull:
-            self._log.warning(
-                f"Blocking on `_cmd_queue.put` as queue full at "
-                f"{self._cmd_queue.qsize()} items",
-            )
-            # Schedule the `put` operation to be executed once there is space in the queue
-            self._loop.create_task(self._cmd_queue.put(command))
-
-    def request(self, request: DataRequest) -> None:
+    def request(self, request: RequestData) -> None:
         """
         Handle the given request.
 
-        If the internal queue is already full then will log a warning and block
-        until queue size reduces.
+        If the internal queue is at or near capacity, it logs a warning (throttled)
+        and schedules an asynchronous `put()` operation. This ensures all messages are
+        eventually enqueued and processed without blocking the caller when the queue is full.
 
         Parameters
         ----------
-        request : DataRequest
+        request : RequestData
             The request to handle.
 
-        Warnings
-        --------
-        This method is not thread-safe and should only be called from the same thread the event
-        loop is running on. Calling it from a different thread may lead to unexpected behavior.
-
         """
-        PyCondition.not_none(request, "request")
-        # Do not allow None through (None is a sentinel value which stops the queue)
-
-        try:
-            self._loop.call_soon_threadsafe(self._req_queue.put_nowait, request)
-        except asyncio.QueueFull:
-            self._log.warning(
-                f"Blocking on `_req_queue.put` as queue full at "
-                f"{self._req_queue.qsize()} items",
-            )
-            # Schedule the `put` operation to be executed once there is space in the queue
-            self._loop.create_task(self._req_queue.put(request))
+        self._req_enqueuer.enqueue(request)
 
     def response(self, response: DataResponse) -> None:
         """
         Handle the given response.
 
-        If the internal queue is already full then will log a warning and block
-        until queue size reduces.
+        If the internal queue is at or near capacity, it logs a warning (throttled)
+        and schedules an asynchronous `put()` operation. This ensures all messages are
+        eventually enqueued and processed without blocking the caller when the queue is full.
 
         Parameters
         ----------
         response : DataResponse
             The response to handle.
 
-        Warnings
-        --------
-        This method is not thread-safe and should only be called from the same thread the event
-        loop is running on. Calling it from a different thread may lead to unexpected behavior.
-
         """
-        PyCondition.not_none(response, "response")
-        # Do not allow None through (None is a sentinel value which stops the queue)
-
-        try:
-            self._loop.call_soon_threadsafe(self._res_queue.put_nowait, response)
-        except asyncio.QueueFull:
-            self._log.warning(
-                f"Blocking on `_res_queue.put` as queue full at "
-                f"{self._res_queue.qsize():_} items",
-            )
-            # Schedule the `put` operation to be executed once there is space in the queue
-            self._loop.create_task(self._res_queue.put(response))
+        self._res_enqueuer.enqueue(response)
 
     def process(self, data: Data) -> None:
         """
-        Process the given data.
+        Process the given data message.
 
-        If the internal queue is already full then will log a warning and block
-        until queue size reduces.
+        If the internal queue is at or near capacity, it logs a warning (throttled)
+        and schedules an asynchronous `put()` operation. This ensures all messages are
+        eventually enqueued and processed without blocking the caller when the queue is full.
 
         Parameters
         ----------
@@ -335,18 +322,7 @@ class LiveDataEngine(DataEngine):
         loop is running on. Calling it from a different thread may lead to unexpected behavior.
 
         """
-        PyCondition.not_none(data, "data")
-        # Do not allow None through (None is a sentinel value which stops the queue)
-
-        try:
-            self._loop.call_soon_threadsafe(self._data_queue.put_nowait, data)
-        except asyncio.QueueFull:
-            self._log.warning(
-                f"Blocking on `_data_queue.put` as queue full at "
-                f"{self._data_queue.qsize():_} items",
-            )
-            # Schedule the `put` operation to be executed once there is space in the queue
-            self._loop.create_task(self._data_queue.put(data))
+        self._data_enqueuer.enqueue(data)
 
     # -- INTERNAL -------------------------------------------------------------------------------------
 
@@ -391,7 +367,7 @@ class LiveDataEngine(DataEngine):
         except asyncio.CancelledError:
             self._log.warning("DataCommand message queue canceled")
         except Exception as e:
-            self._log.error(repr(e))
+            self._log.error(f"{e!r}\n{traceback.format_exc()}")
         finally:
             stopped_msg = "DataCommand message queue stopped"
             if not self._cmd_queue.empty():
@@ -401,20 +377,20 @@ class LiveDataEngine(DataEngine):
 
     async def _run_req_queue(self) -> None:
         self._log.debug(
-            f"DataRequest message queue processing starting (qsize={self.req_qsize()})",
+            f"RequestData message queue processing starting (qsize={self.req_qsize()})",
         )
         try:
             while True:
-                request: DataRequest | None = await self._req_queue.get()
+                request: RequestData | None = await self._req_queue.get()
                 if request is self._sentinel:
                     break
                 self._handle_request(request)
         except asyncio.CancelledError:
-            self._log.warning("DataRequest message queue canceled")
+            self._log.warning("RequestData message queue canceled")
         except Exception as e:
-            self._log.error(repr(e))
+            self._log.error(f"{e!r}\n{traceback.format_exc()}")
         finally:
-            stopped_msg = "DataRequest message queue stopped"
+            stopped_msg = "RequestData message queue stopped"
             if not self._req_queue.empty():
                 self._log.warning(f"{stopped_msg} with {self.req_qsize()} message(s) on queue")
             else:
@@ -433,7 +409,7 @@ class LiveDataEngine(DataEngine):
         except asyncio.CancelledError:
             self._log.warning("DataResponse message queue canceled")
         except Exception as e:
-            self._log.error(repr(e))
+            self._log.error(f"{e!r}\n{traceback.format_exc()}")
         finally:
             stopped_msg = "DataResponse message queue stopped"
             if not self._res_queue.empty():
@@ -452,7 +428,7 @@ class LiveDataEngine(DataEngine):
         except asyncio.CancelledError:
             self._log.warning("Data message queue canceled")
         except Exception as e:
-            self._log.error(repr(e))
+            self._log.error(f"{e!r}\n{traceback.format_exc()}")
         finally:
             stopped_msg = "Data message queue stopped"
             if not self._data_queue.empty():
