@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -15,6 +15,7 @@
 
 import asyncio
 import math
+import traceback
 import uuid
 from asyncio import Queue
 from collections import Counter
@@ -25,6 +26,7 @@ from nautilus_trader.cache.cache import Cache
 from nautilus_trader.common.component import LiveClock
 from nautilus_trader.common.component import MessageBus
 from nautilus_trader.common.enums import LogColor
+from nautilus_trader.common.enums import LogLevel
 from nautilus_trader.config import LiveExecEngineConfig
 from nautilus_trader.core.correctness import PyCondition
 from nautilus_trader.core.datetime import dt_to_unix_nanos
@@ -32,6 +34,8 @@ from nautilus_trader.core.datetime import millis_to_nanos
 from nautilus_trader.core.fsm import InvalidStateTrigger
 from nautilus_trader.core.uuid import UUID4
 from nautilus_trader.execution.engine import ExecutionEngine
+from nautilus_trader.execution.messages import GenerateOrderStatusReports
+from nautilus_trader.execution.messages import GeneratePositionStatusReports
 from nautilus_trader.execution.messages import QueryOrder
 from nautilus_trader.execution.messages import TradingCommand
 from nautilus_trader.execution.reports import ExecutionMassStatus
@@ -39,6 +43,8 @@ from nautilus_trader.execution.reports import ExecutionReport
 from nautilus_trader.execution.reports import FillReport
 from nautilus_trader.execution.reports import OrderStatusReport
 from nautilus_trader.execution.reports import PositionStatusReport
+from nautilus_trader.live.enqueue import ThrottledEnqueuer
+from nautilus_trader.model.book import py_should_handle_own_book_order
 from nautilus_trader.model.enums import LiquiditySide
 from nautilus_trader.model.enums import OrderSide
 from nautilus_trader.model.enums import OrderStatus
@@ -121,14 +127,30 @@ class LiveExecutionEngine(ExecutionEngine):
         self._evt_queue: asyncio.Queue = Queue(maxsize=config.qsize)
         self._inflight_check_retries: Counter[ClientOrderId] = Counter()
 
+        self._cmd_enqueuer: ThrottledEnqueuer[TradingCommand] = ThrottledEnqueuer(
+            qname="cmd_queue",
+            queue=self._cmd_queue,
+            loop=self._loop,
+            clock=self._clock,
+            logger=self._log,
+        )
+        self._evt_enqueuer: ThrottledEnqueuer[OrderEvent] = ThrottledEnqueuer(
+            qname="evt_queue",
+            queue=self._evt_queue,
+            loop=self._loop,
+            clock=self._clock,
+            logger=self._log,
+        )
+
         # Async tasks
         self._cmd_queue_task: asyncio.Task | None = None
         self._evt_queue_task: asyncio.Task | None = None
         self._inflight_check_task: asyncio.Task | None = None
+        self._own_books_audit_task: asyncio.Task | None = None
         self._open_check_task: asyncio.Task | None = None
         self._kill: bool = False
 
-        # Settings
+        # Configuration
         self._reconciliation: bool = config.reconciliation
         self.reconciliation_lookback_mins: int = config.reconciliation_lookback_mins or 0
         self.filter_unclaimed_external_orders: bool = config.filter_unclaimed_external_orders
@@ -137,7 +159,9 @@ class LiveExecutionEngine(ExecutionEngine):
         self.inflight_check_interval_ms: int = config.inflight_check_interval_ms
         self.inflight_check_threshold_ms: int = config.inflight_check_threshold_ms
         self.inflight_check_max_retries: int = config.inflight_check_retries
+        self.own_books_audit_interval_secs: float | None = config.own_books_audit_interval_secs
         self.open_check_interval_secs: float | None = config.open_check_interval_secs
+        self.open_check_open_only: float | None = config.open_check_open_only
         self._inflight_check_threshold_ns: int = millis_to_nanos(self.inflight_check_threshold_ms)
 
         self._log.info(f"{config.reconciliation=}", LogColor.BLUE)
@@ -147,7 +171,9 @@ class LiveExecutionEngine(ExecutionEngine):
         self._log.info(f"{config.inflight_check_interval_ms=}", LogColor.BLUE)
         self._log.info(f"{config.inflight_check_threshold_ms=}", LogColor.BLUE)
         self._log.info(f"{config.inflight_check_retries=}", LogColor.BLUE)
+        self._log.info(f"{config.own_books_audit_interval_secs=}", LogColor.BLUE)
         self._log.info(f"{config.open_check_interval_secs=}", LogColor.BLUE)
+        self._log.info(f"{config.open_check_open_only=}", LogColor.BLUE)
 
         # Register endpoints
         self._msgbus.register(endpoint="ExecEngine.reconcile_report", handler=self.reconcile_report)
@@ -227,6 +253,17 @@ class LiveExecutionEngine(ExecutionEngine):
         """
         return self._inflight_check_task
 
+    def get_own_books_audit_task(self) -> asyncio.Task | None:
+        """
+        Return the own books audit task for the engine.
+
+        Returns
+        -------
+        asyncio.Task or ``None``
+
+        """
+        return self._own_books_audit_task
+
     def get_open_check_task(self) -> asyncio.Task | None:
         """
         Return the open check task for the engine.
@@ -290,54 +327,24 @@ class LiveExecutionEngine(ExecutionEngine):
         command : TradingCommand
             The command to execute.
 
-        Warnings
-        --------
-        This method is not thread-safe and should only be called from the same thread the event
-        loop is running on. Calling it from a different thread may lead to unexpected behavior.
-
         """
-        PyCondition.not_none(command, "command")
-        # Do not allow None through (None is a sentinel value which stops the queue)
-
-        try:
-            self._loop.call_soon_threadsafe(self._cmd_queue.put_nowait, command)
-        except asyncio.QueueFull:
-            self._log.warning(
-                f"Blocking on `_cmd_queue.put` as queue full "
-                f"at {self._cmd_queue.qsize():_} items",
-            )
-            # Schedule the `put` operation to be executed once there is space in the queue
-            self._loop.create_task(self._cmd_queue.put(command))
+        self._cmd_enqueuer.enqueue(command)
 
     def process(self, event: OrderEvent) -> None:
         """
-        Process the given event.
+        Process the given event message.
 
-        If the internal queue is already full then will log a warning and block
-        until queue size reduces.
+        If the internal queue is at or near capacity, it logs a warning (throttled)
+        and schedules an asynchronous `put()` operation. This ensures all messages are
+        eventually enqueued and processed without blocking the caller when the queue is full.
 
         Parameters
         ----------
         event : OrderEvent
             The event to process.
 
-        Warnings
-        --------
-        This method is not thread-safe and should only be called from the same thread the event
-        loop is running on. Calling it from a different thread may lead to unexpected behavior.
-
         """
-        PyCondition.not_none(event, "event")
-
-        try:
-            self._loop.call_soon_threadsafe(self._evt_queue.put_nowait, event)
-        except asyncio.QueueFull:
-            self._log.warning(
-                f"Blocking on `_evt_queue.put` as queue full "
-                f"at {self._evt_queue.qsize():_} items",
-            )
-            # Schedule the `put` operation to be executed once there is space in the queue
-            self._loop.create_task(self._evt_queue.put(event))
+        self._evt_enqueuer.enqueue(event)
 
     # -- INTERNAL -------------------------------------------------------------------------------------
 
@@ -363,6 +370,12 @@ class LiveExecutionEngine(ExecutionEngine):
                 )
                 self._log.debug(f"Scheduled task '{self._inflight_check_task.get_name()}'")
 
+        if self.own_books_audit_interval_secs and not self._own_books_audit_task:
+            self._own_books_audit_task = self._loop.create_task(
+                self._own_books_audit_loop(self.own_books_audit_interval_secs),
+                name="own_books_audit",
+            )
+
         if self.open_check_interval_secs and not self._open_check_task:
             self._open_check_task = self._loop.create_task(
                 self._open_check_loop(self.open_check_interval_secs),
@@ -375,6 +388,11 @@ class LiveExecutionEngine(ExecutionEngine):
             self._inflight_check_task.cancel()
             self._inflight_check_task = None
 
+        if self._own_books_audit_task:
+            self._log.debug(f"Canceling task '{self._own_books_audit_task.get_name()}'")
+            self._own_books_audit_task.cancel()
+            self._own_books_audit_task = None
+
         if self._open_check_task:
             self._log.debug(f"Canceling task '{self._open_check_task.get_name()}'")
             self._open_check_task.cancel()
@@ -397,9 +415,9 @@ class LiveExecutionEngine(ExecutionEngine):
                     break
                 self._execute_command(command)
         except asyncio.CancelledError:
-            self._log.warning("Command message queue canceled")
+            self._log.warning("Canceled task 'run_cmd_queue'")
         except Exception as e:
-            self._log.error(repr(e))
+            self._log.error(f"{e!r}\n{traceback.format_exc()}")
         finally:
             stopped_msg = "Command message queue stopped"
             if not self._cmd_queue.empty():
@@ -418,9 +436,9 @@ class LiveExecutionEngine(ExecutionEngine):
                     break
                 self._handle_event(event)
         except asyncio.CancelledError:
-            self._log.warning("Event message queue canceled")
+            self._log.warning("Canceled task 'run_evt_queue'")
         except Exception as e:
-            self._log.error(repr(e))
+            self._log.error(f"{e!r}\n{traceback.format_exc()}")
         finally:
             stopped_msg = "Event message queue stopped"
             if not self._evt_queue.empty():
@@ -435,9 +453,11 @@ class LiveExecutionEngine(ExecutionEngine):
                 try:
                     await self._check_inflight_orders()
                 except Exception as e:
-                    self._log.error(f"Failed to check in-flight orders: {e}")
+                    self._log.error(
+                        f"Failed to check in-flight orders: {e!r}\n{traceback.format_exc()}",
+                    )
         except asyncio.CancelledError:
-            self._log.debug("In-flight check loop task canceled")
+            self._log.debug("Canceled task 'inflight_check_loop'")
 
     async def _check_inflight_orders(self) -> None:
         self._log.debug("Checking in-flight orders status")
@@ -470,31 +490,69 @@ class LiveExecutionEngine(ExecutionEngine):
                 self._execute_command(query)
                 self._inflight_check_retries[order.client_order_id] += 1
 
+    async def _own_books_audit_loop(self, interval_secs: float) -> None:
+        try:
+            while True:
+                await asyncio.sleep(interval_secs)
+                self._cache.audit_own_order_books()
+        except asyncio.CancelledError:
+            self._log.debug("Canceled task 'own_books_audit_loop'")
+        except Exception as e:
+            self._log.error(f"Error auditing own books: {e}")
+
     async def _open_check_loop(self, interval_secs: float) -> None:
         try:
             while True:
                 await asyncio.sleep(interval_secs)
                 await self._check_open_orders()
         except asyncio.CancelledError:
-            self._log.debug("Open check loop task canceled")
+            self._log.debug("Canceled task 'open_check_loop'")
 
     async def _check_open_orders(self) -> None:
-        self._log.debug("Checking open orders status")
+        try:
+            self._log.debug("Checking open orders status")
 
-        open_order_ids: list[ClientOrderId] = self._cache.client_order_ids_open()
-        open_len = len(open_order_ids)
-        self._log.debug(f"Found {open_len} order{'' if open_len == 1 else 's'} open")
+            open_order_ids: set[ClientOrderId] = self._cache.client_order_ids_open()
+            open_orders: list[Order] = self._cache.orders_open()
+            open_len = len(open_orders)
+            self._log.debug(f"Found {open_len} order{'' if open_len == 1 else 's'} open")
 
-        if not open_order_ids:
-            return  # Nothing further to check
+            # In full-history mode, if there are no cached open orders, skip the venue check
+            # In open-only mode, the venue is always queried regardless of cache state
+            if not open_orders and not self.open_check_open_only:
+                return  # Nothing further to check
 
-        tasks = [c.generate_order_status_reports(open_only=True) for c in self._clients.values()]
-        order_reports_all = await asyncio.gather(*tasks)
-        all_order_reports = [r for reports in order_reports_all for r in reports]
+            if self.open_check_open_only:
+                clients = self._clients.values()
+            else:
+                clients = self.get_clients_for_orders(open_orders)
 
-        for report in all_order_reports:
-            if not report.is_open and report.client_order_id in open_order_ids:
-                self._reconcile_order_report(report, trades=[])
+            tasks = [
+                c.generate_order_status_reports(
+                    GenerateOrderStatusReports(
+                        instrument_id=None,
+                        start=None,
+                        end=None,
+                        open_only=self.open_check_open_only,
+                        command_id=UUID4(),
+                        ts_init=self._clock.timestamp_ns(),
+                        log_receipt_level=LogLevel.DEBUG,
+                    ),
+                )
+                for c in clients
+            ]
+            order_reports_all = await asyncio.gather(*tasks)
+            all_order_reports = [r for reports in order_reports_all for r in reports]
+
+            # For each report, compare the reported open status with our cache
+            # If there's a discrepancy, reconcile the order report
+            for report in all_order_reports:
+                is_in_open_ids = report.client_order_id in open_order_ids
+                if report.is_open != is_in_open_ids:
+                    self._reconcile_order_report(report, trades=[])
+        except Exception as e:
+            # Catch all exception for error visibility in task
+            self._log.error(f"Error in check_open_orders: {e}")
 
     # -- RECONCILIATION -------------------------------------------------------------------------------
 
@@ -573,7 +631,16 @@ class LiveExecutionEngine(ExecutionEngine):
                     self._log.debug(f"Position {instrument_id} for {client_id} already reconciled")
                     continue  # Already reconciled
                 self._log.info(f"{position} pending reconciliation")
-                report_tasks.append(client.generate_position_status_reports(instrument_id))
+                position_status_command = GeneratePositionStatusReports(
+                    instrument_id=instrument_id,
+                    start=None,
+                    end=None,
+                    command_id=UUID4(),
+                    ts_init=self._clock.timestamp_ns(),
+                )
+                report_tasks.append(
+                    client.generate_position_status_reports(position_status_command),
+                )
 
             if report_tasks:
                 # Reconcile specific internal open positions
@@ -730,6 +797,8 @@ class LiveExecutionEngine(ExecutionEngine):
                 return True  # No further reconciliation
             # Add to cache without determining any position ID initially
             self._cache.add_order(order)
+            if self.manage_own_order_books and py_should_handle_own_book_order(order):
+                self._add_own_book_order(order)
 
         instrument: Instrument | None = self._cache.instrument(order.instrument_id)
         if instrument is None:
@@ -972,7 +1041,7 @@ class LiveExecutionEngine(ExecutionEngine):
 
         # Calculate last px
         if order.avg_px is None:
-            last_px: Price = instrument.make_price(report.avg_px)
+            last_px: Price = instrument.make_price(report.avg_px or 0.0)
         else:
             report_cost: float = float(report.avg_px or 0.0) * float(report.filled_qty)
             filled_cost = order.avg_px * float(order.filled_qty)
@@ -988,7 +1057,7 @@ class LiveExecutionEngine(ExecutionEngine):
             client_order_id=order.client_order_id,
             venue_order_id=report.venue_order_id,
             account_id=report.account_id,
-            position_id=PositionId(f"{instrument.id}-EXTERNAL"),
+            position_id=report.venue_position_id or PositionId(f"{instrument.id}-EXTERNAL"),
             trade_id=TradeId(UUID4().value),
             order_side=order.side,
             order_type=order.order_type,
