@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -51,6 +51,7 @@ from nautilus_trader.model.identifiers import TradeId
 from nautilus_trader.model.identifiers import TraderId
 from nautilus_trader.model.identifiers import Venue
 from nautilus_trader.model.identifiers import VenueOrderId
+from nautilus_trader.model.objects import FIXED_PRECISION
 from nautilus_trader.model.objects import AccountBalance
 from nautilus_trader.model.objects import Currency
 from nautilus_trader.model.objects import MarginBalance
@@ -76,6 +77,12 @@ NAUTILUS_PYO3_DATA_TYPES: tuple[type, ...] = (
 )
 
 
+# Convert the given value into the raw integer representation based on the given precision
+# and currently compiled precision mode (128-bit for HIGH_PRECISION or 64-bit).
+def convert_to_raw_int(value, precision: int) -> int:
+    return int(round(float(value), precision) * (10**FIXED_PRECISION))
+
+
 __all__ = [
     "AccountBalance",
     "AccountId",
