@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -70,12 +70,13 @@ class BaseDataCatalog(ABC, metaclass=_CombinedMeta):
         raise NotImplementedError
 
     @abstractmethod
-    def query_last_timestamp(
+    def query_timestamp_bound(
         self,
         data_cls: type,
         instrument_id: str | None = None,
         bar_type: str | None = None,
         ts_column: str = "ts_init",
+        is_last: bool = True,
     ) -> pd.Timestamp | None:
         raise NotImplementedError
 
