@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -32,6 +32,7 @@ from nautilus_trader.model.events import OrderAccepted
 from nautilus_trader.model.events import OrderCanceled
 from nautilus_trader.model.events import OrderCancelRejected
 from nautilus_trader.model.events import OrderDenied
+from nautilus_trader.model.events import OrderEmulated
 from nautilus_trader.model.events import OrderExpired
 from nautilus_trader.model.events import OrderFilled
 from nautilus_trader.model.events import OrderInitialized
@@ -39,41 +40,46 @@ from nautilus_trader.model.events import OrderModifyRejected
 from nautilus_trader.model.events import OrderPendingCancel
 from nautilus_trader.model.events import OrderPendingUpdate
 from nautilus_trader.model.events import OrderRejected
+from nautilus_trader.model.events import OrderReleased
 from nautilus_trader.model.events import OrderSubmitted
 from nautilus_trader.model.events import OrderTriggered
 from nautilus_trader.model.events import OrderUpdated
 
 
+def infer_dtype(dtype_str: str) -> pa.DataType:
+    if dtype_str.startswith("FixedSizeBinary"):
+        return pa.binary(nautilus_pyo3.PRECISION_BYTES)
+    else:
+        return pa.type_for_alias(dtype_str)
+
+
 NAUTILUS_ARROW_SCHEMA = {
     OrderBookDelta: pa.schema(
         [
-            pa.field(k, pa.type_for_alias(v), False)
+            pa.field(k, infer_dtype(v), False)
             for k, v in nautilus_pyo3.OrderBookDelta.get_fields().items()
         ],
     ),
     OrderBookDepth10: pa.schema(
         [
-            pa.field(k, pa.type_for_alias(v), False)
+            pa.field(k, infer_dtype(v), False)
             for k, v in nautilus_pyo3.OrderBookDepth10.get_fields().items()
         ],
     ),
     QuoteTick: pa.schema(
         [
-            pa.field(k, pa.type_for_alias(v), False)
+            pa.field(k, infer_dtype(v), False)
             for k, v in nautilus_pyo3.QuoteTick.get_fields().items()
         ],
     ),
     TradeTick: pa.schema(
         [
-            pa.field(k, pa.type_for_alias(v), False)
+            pa.field(k, infer_dtype(v), False)
             for k, v in nautilus_pyo3.TradeTick.get_fields().items()
         ],
     ),
     Bar: pa.schema(
-        [
-            pa.field(k, pa.type_for_alias(v), False)
-            for k, v in nautilus_pyo3.Bar.get_fields().items()
-        ],
+        [pa.field(k, infer_dtype(v), False) for k, v in nautilus_pyo3.Bar.get_fields().items()],
     ),
     InstrumentClose: pa.schema(
         {
@@ -161,12 +167,12 @@ NAUTILUS_ARROW_SCHEMA = {
             "trigger_instrument_id": pa.string(),
             "contingency_type": pa.string(),
             "order_list_id": pa.string(),
-            "linked_order_ids": pa.string(),
+            "linked_order_ids": pa.binary(),
             "parent_order_id": pa.string(),
             "exec_algorithm_id": pa.string(),
             "exec_algorithm_params": pa.binary(),
             "exec_spawn_id": pa.string(),
-            "tags": pa.string(),
+            "tags": pa.binary(),
             "event_id": pa.string(),
             "ts_init": pa.uint64(),
             "reconciliation": pa.bool_(),
@@ -197,6 +203,17 @@ NAUTILUS_ARROW_SCHEMA = {
             "ts_init": pa.uint64(),
         },
     ),
+    OrderEmulated: pa.schema(
+        {
+            "trader_id": pa.dictionary(pa.int16(), pa.string()),
+            "strategy_id": pa.dictionary(pa.int16(), pa.string()),
+            "instrument_id": pa.dictionary(pa.int64(), pa.string()),
+            "client_order_id": pa.string(),
+            "event_id": pa.string(),
+            "ts_event": pa.uint64(),
+            "ts_init": pa.uint64(),
+        },
+    ),
     OrderSubmitted: pa.schema(
         {
             "trader_id": pa.dictionary(pa.int16(), pa.string()),
@@ -322,6 +339,18 @@ NAUTILUS_ARROW_SCHEMA = {
             "reconciliation": pa.bool_(),
         },
     ),
+    OrderReleased: pa.schema(
+        {
+            "trader_id": pa.dictionary(pa.int16(), pa.string()),
+            "strategy_id": pa.dictionary(pa.int16(), pa.string()),
+            "instrument_id": pa.dictionary(pa.int64(), pa.string()),
+            "client_order_id": pa.string(),
+            "released_price": pa.string(),
+            "event_id": pa.string(),
+            "ts_event": pa.uint64(),
+            "ts_init": pa.uint64(),
+        },
+    ),
     OrderModifyRejected: pa.schema(
         {
             "trader_id": pa.dictionary(pa.int16(), pa.string()),
