@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -39,9 +39,11 @@ from nautilus_trader.common.component import LogGuard
 from nautilus_trader.common.component import MessageBus
 from nautilus_trader.common.component import TestClock
 from nautilus_trader.common.component import init_logging
+from nautilus_trader.common.component import is_backtest_force_stop
 from nautilus_trader.common.component import is_logging_initialized
 from nautilus_trader.common.component import log_header
 from nautilus_trader.common.component import register_component_clock
+from nautilus_trader.common.component import set_backtest_force_stop
 from nautilus_trader.common.component import set_logging_pyo3
 from nautilus_trader.common.config import InvalidConfiguration
 from nautilus_trader.common.config import msgspec_encoding_hook
@@ -153,7 +155,6 @@ class NautilusKernel:
         self._trader_id: TraderId = trader_id
         self._machine_id: str = socket.gethostname()
         self._instance_id: UUID4 = config.instance_id or UUID4()
-        self._ts_created: int = time.time_ns()
 
         # Components
         if self._environment == Environment.BACKTEST:
@@ -165,6 +166,11 @@ class NautilusKernel:
                 f"environment {self._environment} not recognized",  # pragma: no cover (design-time error)
             )
 
+        self._ts_created: int = self._clock.timestamp_ns()
+        self._ts_started: int | None = None
+        self._ts_shutdown: int | None = None
+        ts_build = time.time_ns()
+
         register_component_clock(self._instance_id, self._clock)
 
         # Initialize logging system
@@ -196,7 +202,7 @@ class NautilusKernel:
                     # Initialize logging for sync Rust and Python
                     self._log_guard = nautilus_pyo3.init_logging(
                         trader_id=nautilus_pyo3.TraderId(self._trader_id.value),
-                        instance_id=nautilus_pyo3.UUID4(self._instance_id.value),
+                        instance_id=nautilus_pyo3.UUID4.from_str(self._instance_id.value),
                         level_stdout=nautilus_pyo3.LogLevel(logging.log_level),
                         level_file=nautilus_pyo3.LogLevel(logging.log_level_file or "OFF"),
                         directory=logging.log_directory,
@@ -209,7 +215,7 @@ class NautilusKernel:
                     nautilus_pyo3.log_header(
                         trader_id=nautilus_pyo3.TraderId(self._trader_id.value),
                         machine_id=self._machine_id,
-                        instance_id=nautilus_pyo3.UUID4(self._instance_id.value),
+                        instance_id=nautilus_pyo3.UUID4.from_str(self._instance_id.value),
                         component=name,
                     )
                 else:
@@ -269,7 +275,7 @@ class NautilusKernel:
         elif config.message_bus.database.type == "redis":
             self._msgbus_db = nautilus_pyo3.RedisMessageBusDatabase(
                 trader_id=nautilus_pyo3.TraderId(self._trader_id.value),
-                instance_id=nautilus_pyo3.UUID4(self._instance_id.value),
+                instance_id=nautilus_pyo3.UUID4.from_str(self._instance_id.value),
                 config_json=msgspec.json.encode(config.message_bus, enc_hook=msgspec_encoding_hook),
             )
         else:
@@ -334,6 +340,7 @@ class NautilusKernel:
             msgbus=self._msgbus,
             cache=self._cache,
             clock=self._clock,
+            config=config.portfolio,
         )
 
         ########################################################################
@@ -463,13 +470,7 @@ class NautilusKernel:
                 config=self._config.controller,
                 trader=self._trader,
             )
-            assert self._controller is not None  # Type checking
-            self._controller.register_base(
-                portfolio=self._portfolio,
-                msgbus=self._msgbus,
-                cache=self._cache,
-                clock=self._clock,
-            )
+            self._trader.add_actor(self._controller)
 
         # Set up stream writer
         self._writer: StreamingFeatherWriter | None = None
@@ -511,7 +512,11 @@ class NautilusKernel:
             exec_algorithm: ExecAlgorithm = ExecAlgorithmFactory.create(exec_algorithm_config)
             self._trader.add_exec_algorithm(exec_algorithm)
 
-        build_time_ms = nanos_to_millis(time.time_ns() - self.ts_created)
+        # State flags
+        self._is_running = False
+        self._is_stopping = False
+
+        build_time_ms = nanos_to_millis(time.time_ns() - ts_build)
         self._log.info(f"Initialized in {build_time_ms}ms")
 
     def __del__(self) -> None:
@@ -573,6 +578,16 @@ class NautilusKernel:
             self._log.warning(f"Received {command!r} not for this trader {self.trader_id}")
             return
 
+        if self._environment == Environment.BACKTEST and is_backtest_force_stop():
+            return  # Backtest has already been force stopped
+
+        if not self._is_running:
+            self._log.warning(f"Received {command!r} when not running")
+            return
+
+        if self._is_stopping:
+            return  # Already stopping
+
         self._log.info(f"Received {command!r}, shutting down...", LogColor.BLUE)
 
         if self._loop:
@@ -580,6 +595,10 @@ class NautilusKernel:
         else:
             self.stop()
 
+        if self._environment == Environment.BACKTEST:
+            set_backtest_force_stop(True)
+            self._log.debug("Set backtest FORCE_STOP")
+
     @property
     def environment(self) -> Environment:
         """
@@ -683,11 +702,35 @@ class NautilusKernel:
 
         Returns
         -------
-        uint64_t
+        int
 
         """
         return self._ts_created
 
+    @property
+    def ts_started(self) -> int | None:
+        """
+        Return the UNIX timestamp (nanoseconds) when the kernel was last started.
+
+        Returns
+        -------
+        int or ``None``
+
+        """
+        return self._ts_started
+
+    @property
+    def ts_shutdown(self) -> int | None:
+        """
+        Return the UNIX timestamp (nanoseconds) when the kernel was last shutdown.
+
+        Returns
+        -------
+        int or ``None``
+
+        """
+        return self._ts_shutdown
+
     @property
     def load_state(self) -> bool:
         """
@@ -893,11 +936,24 @@ class NautilusKernel:
         """
         return self._log_guard
 
+    def is_running(self) -> bool:
+        """
+        Return whether the kernel is running.
+
+        Returns
+        -------
+        bool
+
+        """
+        return self._is_running
+
     def start(self) -> None:
         """
         Start the Nautilus system kernel.
         """
         self._log.info("STARTING")
+        self._ts_started = self._clock.timestamp_ns()
+        self._is_running = True
 
         self._start_engines()
         self._connect_clients()
@@ -905,9 +961,6 @@ class NautilusKernel:
         self._initialize_portfolio()
         self._trader.start()
 
-        if self._controller:
-            self._controller.start()
-
     async def start_async(self) -> None:
         """
         Start the Nautilus system kernel in an asynchronous context with an event loop.
@@ -922,6 +975,8 @@ class NautilusKernel:
             raise RuntimeError("no event loop has been assigned to the kernel")
 
         self._log.info("STARTING")
+        self._ts_started = self._clock.timestamp_ns()
+        self._is_running = True
 
         self._register_executor()
         self._start_engines()
@@ -941,20 +996,15 @@ class NautilusKernel:
 
         self._trader.start()
 
-        if self._controller:
-            self._controller.start()
-
     def stop(self) -> None:
         """
         Stop the Nautilus system kernel.
         """
         self._log.info("STOPPING")
+        self._is_stopping = True
 
         self._stop_clients()
 
-        if self._controller:
-            self._controller.stop()
-
         if self._trader.is_running:
             self._trader.stop()
 
@@ -968,6 +1018,9 @@ class NautilusKernel:
         self._flush_writer()
 
         self._log.info("STOPPED")
+        self._is_running = False
+        self._is_stopping = False
+        self._ts_shutdown = self._clock.timestamp_ns()
 
     async def stop_async(self) -> None:
         """
@@ -987,6 +1040,7 @@ class NautilusKernel:
             raise RuntimeError("no event loop has been assigned to the kernel")
 
         self._log.info("STOPPING")
+        self._is_stopping = True
 
         if self._trader.is_running:
             self._trader.stop()
@@ -1005,6 +1059,9 @@ class NautilusKernel:
         self._flush_writer()
 
         self._log.info("STOPPED")
+        self._is_running = False
+        self._is_stopping = False
+        self._ts_shutdown = self._clock.timestamp_ns()
 
     def dispose(self) -> None:
         """
@@ -1034,49 +1091,115 @@ class NautilusKernel:
         if self._writer:
             self._writer.close()
 
-    def cancel_all_tasks(self) -> None:
+    def cancel_all_tasks(self) -> None:  # noqa: C901 (too complex)
         """
         Cancel all tasks currently running for the Nautilus kernel.
 
+        This method handles task cancellation in both synchronous and asynchronous contexts:
+        - 1. All pending tasks are identified and given a cancellation signal.
+        - 2. The kernel waits until the shutdown timeout for tasks to conclude or raise exceptions.
+        - 3. Any tasks that remain unresponsive or encounter errors are logged.
+        - 4. If the event loop is still active, the cancellation proceeds asynchronously;
+        otherwise, it's handled immediately.
+
         Raises
         ------
         RuntimeError
             If no event loop has been assigned to the kernel.
 
+        Notes
+        -----
+        - Tasks that don't respond to cancellation within timeout are logged but not forcibly terminated.
+        - CancelledError exceptions are expected and handled silently.
+        - Other exceptions from tasks are logged with full stack traces.
+        - If the loop is closed, method exits early with a warning.
+
         """
         if self.loop is None:
             raise RuntimeError("no event loop has been assigned to the kernel")
 
-        to_cancel = asyncio.tasks.all_tasks(self.loop)
-        if not to_cancel:
-            self._log.info("All tasks canceled")
+        if self.loop.is_closed():
+            self._log.warning("Event loop already closed; cannot cancel tasks")
             return
 
-        for task in to_cancel:
-            self._log.warning(f"Canceling pending task '{task.get_name()}'")
+        # Get all tasks except the current one
+        current_task = asyncio.current_task(self.loop)
+        pending_tasks = [
+            task
+            for task in asyncio.all_tasks(self.loop)
+            if task is not current_task and not task.done()
+        ]
+
+        if not pending_tasks:
+            self._log.info("No pending tasks to cancel")
+            return
+
+        # Log tasks that are about to be cancelled
+        for task in pending_tasks:
+            self._log.info(f"Canceling pending task '{task.get_name()}' (id={id(task)})")
             task.cancel()
 
-        if self.loop and self.loop.is_running():
-            self._log.warning("Event loop still running during `cancel_all_tasks`")
-            return
-
-        finish_all_tasks: asyncio.Future = asyncio.tasks.gather(*to_cancel)
-        self.loop.run_until_complete(finish_all_tasks)
-
-        self._log.debug(f"{finish_all_tasks}")
-
-        for task in to_cancel:  # pragma: no cover
-            if task.cancelled():
-                continue
-            if task.exception() is not None:
-                self.loop.call_exception_handler(
-                    {
-                        "message": "unhandled exception during `asyncio.run()` shutdown",
-                        "exception": task.exception(),
-                        "task": task,
-                    },
+        async def _cancel_and_wait_for_tasks():
+            try:
+                done, still_pending = await asyncio.wait(
+                    pending_tasks,
+                    timeout=self._config.timeout_shutdown,
+                    return_when=asyncio.ALL_COMPLETED,
                 )
 
+                # Handle any tasks that didn't complete within the timeout
+                if still_pending:
+                    self._log.warning(
+                        f"{len(still_pending)} tasks still pending after {self._config.timeout_shutdown}s timeout:",
+                    )
+                    for t in still_pending:
+                        self._log.warning(f"Task '{t.get_name()}' (id={id(t)}) still pending")
+
+                # Log any exceptions from the completed tasks
+                for d in done:
+                    try:
+                        exc = d.exception()
+                        if exc and not isinstance(exc, asyncio.CancelledError):
+                            self._log.error(
+                                f"Task '{d.get_name()}' raised unexpected exception: {exc}",
+                                exc_info=exc,
+                            )
+                    except asyncio.CancelledError:
+                        pass  # This is expected for cancelled tasks
+            except Exception as e:
+                self._log.error(
+                    f"Error during task cleanup: {e}",
+                    exc_info=True,
+                )
+
+        if self.loop.is_running():
+            # If the loop is already running, schedule the cleanup and run asynchronously
+            self._log.info("Event loop still running; scheduling task cleanup")
+            cleanup_task = self.loop.create_task(_cancel_and_wait_for_tasks())
+            cleanup_task.add_done_callback(
+                lambda t: self._log.info(
+                    (
+                        "Task cleanup completed"
+                        if not t.exception()
+                        else f"Task cleanup failed: {t.exception()}"
+                    ),
+                ),
+            )
+        else:
+            try:
+                # If the loop isn't running, we can block until cleanup completes
+                self.loop.run_until_complete(_cancel_and_wait_for_tasks())
+            except RuntimeError as e:
+                if "Event loop stopped before Future completed" in str(e):
+                    self._log.warning(
+                        "Loop stopped during cleanup; some tasks may not "
+                        "be properly canceled or awaited",
+                    )
+                else:
+                    raise
+
+        self._log.info("Task cancellation completed")
+
     def _register_executor(self) -> None:
         for actor in self.trader.actors():
             actor.register_executor(self._loop, self._executor)
