@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -13,23 +13,25 @@
 #  limitations under the License.
 # -------------------------------------------------------------------------------------------------
 
-from datetime import datetime
+
 from pathlib import Path
-from typing import Any, Literal
+from typing import Literal
 
 from nautilus_trader.cache.cache import Cache
 from nautilus_trader.common.component import Clock
 from nautilus_trader.common.component import MessageBus
-from nautilus_trader.core.uuid import UUID4
 from nautilus_trader.data.client import MarketDataClient
+from nautilus_trader.data.messages import RequestBars
+from nautilus_trader.data.messages import RequestInstrument
+from nautilus_trader.data.messages import RequestInstruments
+from nautilus_trader.data.messages import RequestQuoteTicks
+from nautilus_trader.data.messages import RequestTradeTicks
 from nautilus_trader.model.data import Bar
-from nautilus_trader.model.data import BarType
 from nautilus_trader.model.data import QuoteTick
 from nautilus_trader.model.data import TradeTick
 from nautilus_trader.model.identifiers import ClientId
-from nautilus_trader.model.identifiers import InstrumentId
 from nautilus_trader.model.identifiers import Venue
-from nautilus_trader.model.instruments.base import Instrument
+from nautilus_trader.model.instruments import Instrument
 from nautilus_trader.persistence.catalog.parquet import ParquetDataCatalog
 from nautilus_trader.persistence.catalog.singleton import clear_singleton_instances
 from nautilus_trader.persistence.wranglers import QuoteTickDataWrangler
@@ -78,58 +80,30 @@ class MockMarketDataClient(MarketDataClient):
         self.trade_ticks: list[TradeTick] = []
         self.bars: list[Bar] = []
 
-    def request_instrument(
-        self,
-        instrument_id: InstrumentId,
-        correlation_id: UUID4,
-        start: datetime | None = None,
-        end: datetime | None = None,
-        params: dict[str, Any] | None = None,
-    ) -> None:
-        self._handle_instrument(self.instrument, correlation_id, params)
+    def request_instrument(self, request: RequestInstrument) -> None:
+        self._handle_instrument(self.instrument, request.id, request.params)
 
-    def request_instruments(
-        self,
-        venue: Venue,
-        correlation_id: UUID4,
-        start: datetime | None = None,
-        end: datetime | None = None,
-        params: dict[str, Any] | None = None,
-    ) -> None:
-        self._handle_instruments(venue, self.instruments, correlation_id, params)
+    def request_instruments(self, request: RequestInstruments) -> None:
+        self._handle_instruments(request.venue, self.instruments, request.id, request.params)
 
-    def request_quote_ticks(
-        self,
-        instrument_id: InstrumentId,
-        limit: int,
-        correlation_id: UUID4,
-        start: datetime | None = None,
-        end: datetime | None = None,
-        params: dict[str, Any] | None = None,
-    ) -> None:
-        self._handle_quote_ticks(instrument_id, self.quote_ticks, correlation_id, params)
+    def request_quote_ticks(self, request: RequestQuoteTicks) -> None:
+        self._handle_quote_ticks(
+            request.instrument_id,
+            self.quote_ticks,
+            request.id,
+            request.params,
+        )
 
-    def request_trade_ticks(
-        self,
-        instrument_id: InstrumentId,
-        limit: int,
-        correlation_id: UUID4,
-        start: datetime | None = None,
-        end: datetime | None = None,
-        params: dict[str, Any] | None = None,
-    ) -> None:
-        self._handle_trade_ticks(instrument_id, self.trade_ticks, correlation_id, params)
+    def request_trade_ticks(self, request: RequestTradeTicks) -> None:
+        self._handle_trade_ticks(
+            request.instrument_id,
+            self.trade_ticks,
+            request.id,
+            request.params,
+        )
 
-    def request_bars(
-        self,
-        bar_type: BarType,
-        limit: int,
-        correlation_id: UUID4,
-        start: datetime | None = None,
-        end: datetime | None = None,
-        params: dict[str, Any] | None = None,
-    ) -> None:
-        self._handle_bars(bar_type, self.bars, None, correlation_id, params)
+    def request_bars(self, request: RequestBars) -> None:
+        self._handle_bars(request.bar_type, self.bars, None, request.id, request.params)
 
 
 _AUDUSD_SIM = TestInstrumentProvider.default_fx_ccy("AUD/USD")
