@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -18,10 +18,12 @@ from decimal import Decimal
 import pandas as pd
 import pytz
 
+from nautilus_trader.core import nautilus_pyo3
 from nautilus_trader.core.nautilus_pyo3 import AssetClass
 from nautilus_trader.core.nautilus_pyo3 import BettingInstrument
 from nautilus_trader.core.nautilus_pyo3 import BinaryOption
 from nautilus_trader.core.nautilus_pyo3 import CryptoFuture
+from nautilus_trader.core.nautilus_pyo3 import CryptoOption
 from nautilus_trader.core.nautilus_pyo3 import CryptoPerpetual
 from nautilus_trader.core.nautilus_pyo3 import Currency
 from nautilus_trader.core.nautilus_pyo3 import CurrencyPair
@@ -30,21 +32,21 @@ from nautilus_trader.core.nautilus_pyo3 import FuturesContract
 from nautilus_trader.core.nautilus_pyo3 import FuturesSpread
 from nautilus_trader.core.nautilus_pyo3 import InstrumentId
 from nautilus_trader.core.nautilus_pyo3 import Money
+from nautilus_trader.core.nautilus_pyo3 import OptionContract
 from nautilus_trader.core.nautilus_pyo3 import OptionKind
-from nautilus_trader.core.nautilus_pyo3 import OptionsContract
-from nautilus_trader.core.nautilus_pyo3 import OptionsSpread
+from nautilus_trader.core.nautilus_pyo3 import OptionSpread
 from nautilus_trader.core.nautilus_pyo3 import Price
 from nautilus_trader.core.nautilus_pyo3 import Quantity
 from nautilus_trader.core.nautilus_pyo3 import Symbol
 from nautilus_trader.core.nautilus_pyo3 import Venue
-from nautilus_trader.test_kit.rust.types_pyo3 import TestTypesProviderPyo3
 
 
-_USD = TestTypesProviderPyo3.currency_usd()
-_USDC = TestTypesProviderPyo3.currency_usdc()
-_USDT = TestTypesProviderPyo3.currency_usdt()
-_BTC = TestTypesProviderPyo3.currency_btc()
-_ETH = TestTypesProviderPyo3.currency_eth()
+_GBP = nautilus_pyo3.Currency.from_str("GBP")
+_USD = nautilus_pyo3.Currency.from_str("USD")
+_USDC = nautilus_pyo3.Currency.from_str("USDC")
+_USDT = nautilus_pyo3.Currency.from_str("USDT")
+_BTC = nautilus_pyo3.Currency.from_str("BTC")
+_ETH = nautilus_pyo3.Currency.from_str("ETH")
 
 
 class TestInstrumentProviderPyo3:
@@ -116,6 +118,7 @@ class TestInstrumentProviderPyo3:
             size_precision=2,  # BETFAIR_QUANTITY_PRECISION,
             price_increment=Price.from_str("0.01"),
             size_increment=Quantity.from_str("0.01"),
+            min_notional=Money(1, _GBP),
             maker_fee=Decimal(0),  # TBD
             taker_fee=Decimal(0),  # TBD
             ts_event=0,
@@ -299,19 +302,50 @@ class TestInstrumentProviderPyo3:
             max_quantity=Quantity.from_str("9000"),
             min_quantity=Quantity.from_str("0.00001"),
             max_notional=None,
-            min_notional=Money(10.0, TestTypesProviderPyo3.currency_usdt()),
+            min_notional=Money(10.0, _USDT),
             max_price=Price.from_str("1000000.0"),
             min_price=Price.from_str("0.01"),
             ts_event=0,
             ts_init=0,
         )
 
+    @staticmethod
+    def btcusd_option_deribit() -> CryptoOption:
+        return CryptoOption(
+            id=InstrumentId.from_str("BTC-13JAN23-16000-P.DERIBIT"),
+            raw_symbol=Symbol("BTC-13JAN23-16000-P"),
+            underlying=_BTC,
+            quote_currency=_USD,
+            settlement_currency=_BTC,
+            is_inverse=False,
+            option_kind=OptionKind.PUT,
+            strike_price=Price.from_str("16000.00"),
+            activation_ns=1671696002000000000,
+            expiration_ns=1673596800000000000,
+            price_precision=2,
+            size_precision=1,
+            price_increment=Price.from_str("0.01"),
+            size_increment=Quantity.from_str("0.1"),
+            maker_fee=Decimal("0.0003"),
+            taker_fee=Decimal("0.0003"),
+            margin_init=Decimal("0"),  # TBD
+            margin_maint=Decimal("0"),  # TBD
+            max_quantity=Quantity.from_str("9000"),
+            min_quantity=Quantity.from_str("0.1"),
+            max_notional=None,
+            min_notional=Money.from_str("10.00 USD"),
+            max_price=None,
+            min_price=None,
+            ts_event=0,
+            ts_init=0,
+        )
+
     @staticmethod
     def btcusdt_binance() -> CurrencyPair:
         return CurrencyPair(
             id=InstrumentId.from_str("BTCUSDT.BINANCE"),
             raw_symbol=Symbol("BTCUSDT"),
-            base_currency=TestTypesProviderPyo3.currency_btc(),
+            base_currency=_BTC,
             quote_currency=_USDT,
             price_precision=2,
             size_precision=6,
@@ -352,12 +386,12 @@ class TestInstrumentProviderPyo3:
     def aapl_option(
         activation: pd.Timestamp | None = None,
         expiration: pd.Timestamp | None = None,
-    ) -> OptionsContract:
+    ) -> OptionContract:
         if activation is None:
             activation = pd.Timestamp("2021-9-17", tz=pytz.utc)
         if expiration is None:
             expiration = pd.Timestamp("2021-12-17", tz=pytz.utc)
-        return OptionsContract(
+        return OptionContract(
             id=InstrumentId.from_str("AAPL211217C00150000.OPRA"),
             raw_symbol=Symbol("AAPL211217C00150000"),
             asset_class=AssetClass.EQUITY,
@@ -442,15 +476,15 @@ class TestInstrumentProviderPyo3:
         )
 
     @staticmethod
-    def options_spread(
+    def option_spread(
         activation: pd.Timestamp | None = None,
         expiration: pd.Timestamp | None = None,
-    ) -> OptionsSpread:
+    ) -> OptionSpread:
         if activation is None:
             activation = pd.Timestamp("2023-11-06T20:54:07", tz=pytz.utc)
         if expiration is None:
             expiration = pd.Timestamp("2024-02-23T22:59:00", tz=pytz.utc)
-        return OptionsSpread(
+        return OptionSpread(
             id=InstrumentId.from_str("UD:U$: GN 2534559.GLBX"),
             raw_symbol=Symbol("UD:U$: GN 2534559"),
             asset_class=AssetClass.FX,
