@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -14,7 +14,6 @@
 # -------------------------------------------------------------------------------------------------
 
 import json
-from datetime import datetime
 from os import PathLike
 from typing import Any
 
@@ -49,6 +48,7 @@ from nautilus_trader.model.identifiers import Symbol
 from nautilus_trader.model.identifiers import TradeId
 from nautilus_trader.model.identifiers import Venue
 from nautilus_trader.model.instruments import Instrument
+from nautilus_trader.model.objects import FIXED_SCALAR
 from nautilus_trader.model.objects import Price
 from nautilus_trader.model.objects import Quantity
 from nautilus_trader.persistence.wranglers import BarDataWrangler
@@ -58,7 +58,9 @@ from nautilus_trader.test_kit.providers import TestInstrumentProvider
 from nautilus_trader.test_kit.stubs.identifiers import TestIdStubs
 
 
-UNIX_EPOCH = datetime(1970, 1, 1, 0, 0, 0, 0, tzinfo=pytz.utc)
+# UNIX epoch is the UTC time at midnight on 1970-01-01
+# https://en.wikipedia.org/wiki/Unix_time
+UNIX_EPOCH = pd.Timestamp("1970-01-01", tzinfo=pytz.utc)
 
 
 class TestDataStubs:
@@ -525,7 +527,7 @@ class TestDataStubs:
                         "op": "update",
                         "order": BookOrder(
                             price=Price(row[side], precision=6),
-                            size=Quantity(1e9, precision=2),
+                            size=Quantity(FIXED_SCALAR, precision=2),
                             side=order_side,
                             order_id=0,
                         ),
